@@ -1,4 +1,22 @@
-/* $Id: acconfig.h,v 1.2 2000/12/29 11:39:58 gray Exp $ */
+/* This file is part of GNU RADIUS.
+ * Copyright (C) 2000,2001, Sergey Poznyakoff
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ */
+/* $Id: acconfig.h,v 1.4 2001/03/07 10:42:10 gray Exp $ */
 
 #undef PW_AUTH_UDP_PORT
 #undef PW_ACCT_UDP_PORT
@@ -31,6 +49,12 @@
 #endif
 #define N_(s) s
 
+/* ************************************************************************** */
+/* data types that might be missed */
+#undef off_t
+#undef u_char
+#undef u_int
+#undef u_long
 /* ************************************************************************** */
 
 #undef HAVE_SYS_ERRLIST
