@@ -17,7 +17,7 @@
  *
  */
 /*
- *	@(#) $Id: radiusd.h,v 1.4 2001/01/26 11:48:12 gray Exp $
+ *	@(#) $Id: radiusd.h,v 1.12 2001/03/12 11:13:40 gray Exp $
  */
 
 #include <stdio.h>
@@ -26,7 +26,6 @@
 #include <radpaths.h>
 #include <mem.h>
 #include <log.h>
-#include <ippool.h>
 
 #define NITEMS(a) sizeof(a)/sizeof((a)[0])
 
@@ -35,11 +34,11 @@ struct radutmp; /* declared in radutmp.h */
 
 
 typedef struct {
-	int delayed_hup_wait;
+	unsigned delayed_hup_wait;
 	int checkrad_assume_logged;
 	int max_requests;
-	char exec_user[32];
-	char exec_group[32];
+	char *exec_user;
+	char *exec_group;
 } Config;
 
 typedef struct {
@@ -166,7 +165,7 @@ typedef struct auth_req {
 	int			data_len;       /* Length of raw data */
 	int                     data_alloced;   /* Was the data malloced */
         /* Proxy support fields */
-	u_char			*realm;         /* stringobj, actually */
+	char			*realm;         /* stringobj, actually */
 	int			validated;	/* Already md5 checked */
 	UINT4			server_ipaddr;
 	UINT4			server_id;
@@ -198,6 +197,7 @@ typedef struct proxy_state {
 #ifdef USE_SNMP
 struct nas_stat;
 #endif
+struct radck_arg;
 
 typedef struct nas {
 	struct nas		*next;
@@ -205,7 +205,7 @@ typedef struct nas {
 	char			longname[MAX_LONGNAME+1];
 	char			shortname[MAX_SHORTNAME+1];
 	char			nastype[MAX_DICTNAME+1];
-	IP_POOL                 *ip_pool;
+	struct radck_arg        *args;
 #ifdef USE_SNMP
 	struct nas_stat         *nas_stat;
 #endif	
@@ -219,8 +219,21 @@ typedef struct realm {
 	int			auth_port;
 	int			acct_port;
 	int			striprealm;
+	int                     maxlogins;
 } REALM;
 
+/*
+ * Internal representation of a user's profile
+ */
+typedef struct user_symbol {
+	struct user_symbol *next;
+	char *name;
+	int lineno;
+	int ordnum;
+	VALUE_PAIR *check;
+	VALUE_PAIR *reply;
+} User_symbol;
+
 struct keyword {
 	char *name;
 	int tok;
@@ -347,11 +360,14 @@ enum {
 
 #define VENDOR(x) (x >> 16)
 
-/* DBM usage flags
+/*
+ * Authentication results
  */
-#define DBM_NEVER                       0
-#define DBM_ONLY                        1
-#define DBM_ALSO                        2
+#define AUTH_OK      0 /* OK */
+#define AUTH_FAIL    1 /* Password fail */
+#define AUTH_NOUSER  2 /* No such user  */
+#define AUTH_REJECT  3 /* Rejected */
+
 
 /*
  *	Global variables.
@@ -399,7 +415,7 @@ extern int              notify_port;
 /* acct.c */
 int		rad_accounting(AUTH_REQ *, int);
 int		rad_account_transfer(AUTH_REQ *, int);
-int		rad_accounting_orig(AUTH_REQ *, int, char *);
+int		rad_accounting_orig(AUTH_REQ *, int);
 int		rad_account_slice(AUTH_REQ *, int);
 int		radzap(UINT4 nas, int port, char *user, time_t t);
 char		*uue(void *);
@@ -430,7 +446,6 @@ void		md5_calc(u_char *, u_char *, u_int);
 
 /* radiusd.c */
 void		debug_pair(char *, VALUE_PAIR *);
-int		log_err (char *);
 void		sig_cleanup(int);
 int             server_type();
 int             stat_request_list(int (*report)());
@@ -438,6 +453,7 @@ void *          scan_request_list(int type, int (*handler)(), void *closure);
 int             set_nonblocking(int fd);
 int             master_process();
 int             rad_flush_queues();
+void            schedule_restart();
 
 /* util.c */
 char *		ip_hostname (UINT4);
@@ -480,10 +496,11 @@ char		*client_name(UINT4 ipno);
 int		read_clients_file(char *);
 REALM		*realm_find(char *);
 NAS		*nas_find(UINT4 ipno);
+NAS             *nas_by_name(char *name);
 char		*nas_name(UINT4 ipno);
 char		*nas_name2(AUTH_REQ *r);
 int		read_naslist_file(char *);
-int		reload_config_file(int);
+int		reload_config_file(enum reload_what);
 int		presufcmp(VALUE_PAIR *check, char *name, char *rest);
 void		pairmove(VALUE_PAIR **to, VALUE_PAIR **from);
 void		pairmove2(VALUE_PAIR **to, VALUE_PAIR **from, int attr);
@@ -501,7 +518,8 @@ void		version();
 
 /* pam.c */
 #ifdef USE_PAM
-int		pam_pass(char *name, char *passwd, const char *pamauth);
+int		pam_pass(char *name, char *passwd, const char *pamauth,
+			 char **reply_msg);
 # define PAM_DEFAULT_TYPE    "radius"
 #endif
 
@@ -518,14 +536,14 @@ int		rad_authenticate (AUTH_REQ *, int);
 int  rad_check_password(AUTH_REQ *authreq, int activefd,
 			VALUE_PAIR *check_item,
 			VALUE_PAIR *namepair,
-			char *pw_digest, char **user_msg, char *userpass);
+			u_char *pw_digest, char **user_msg, char *userpass);
 
 /* exec.c */
 int		radius_exec_program(char *, VALUE_PAIR *,
 				    VALUE_PAIR **, int, char **user_msg);
 
 /* menu.c */
-void process_menu(AUTH_REQ *authreq, int fd, char *pw_digest);
+void process_menu(AUTH_REQ *authreq, int fd, u_char *pw_digest);
 char * get_menu(char *menu_name);
 
 /* fixalloc.c */
@@ -548,11 +566,9 @@ int		timestr_match(char *, time_t);
 
 /* notify.c */
 int notify(char *login, int what, long *ttl_ptr);
+int notify_acct(char *login, int what);
 int timetolive(char *user_name, long *ttl);
 
-/* ippool.c */
-VALUE_PAIR * alloc_ip_pair(char *name, AUTH_REQ *authreq);
-
 /* shmem.c */
 int shmem_alloc(unsigned size);
 void shmem_free();
@@ -582,7 +598,7 @@ int read_raddb_file(char *name, int vital, int fcnt, int (*fun)(),
 int xlat_keyword(struct keyword *kw, char *str, int def);
 
 /* mem.c */
-void *emalloc(unsigned);
+void *emalloc(size_t);
 void efree(void*);
 char *estrdup(char *);
 
@@ -602,7 +618,8 @@ int user_gettime(char *valstr, struct tm *tm);
 
 /* snmpserver.c */
 struct sockaddr_in;
-struct snmp_req * rad_snmp_respond(char *buf, int len, struct sockaddr_in *sa);
+struct snmp_req * rad_snmp_respond(u_char *buf, int len,
+				   struct sockaddr_in *sa);
 int snmp_req_cmp(struct snmp_req *a, struct snmp_req *b);
 void snmp_req_free(struct snmp_req  *req);
 void snmp_req_drop(int type, struct snmp_req *req, char *status_str);
