@@ -52,10 +52,12 @@ typedef struct {
 	char     *acct_db;
 	char     *auth_db;
 	char     *auth_query;
+	char     *group_query;
 	char     *acct_start_query;
 	char     *acct_stop_query;
 	char     *acct_nasup_query;
 	char     *acct_nasdown_query;
+	char     *acct_keepalive_query;
 	BUFFER   buf;
 	int      keepopen;
 	time_t   idle_timeout;
@@ -69,22 +71,29 @@ extern SQL_cfg sql_cfg;
 
 int rad_sql_init();
 void rad_sql_acct(AUTH_REQ *req);
-int rad_sql_pass(AUTH_REQ *req, char *passwd);
+int rad_sql_pass(AUTH_REQ *req, char *data, char *passwd);
 void rad_sql_check_connect(int type);
 void rad_sql_need_reconnect(int type);
 int rad_sql_setup(int type, qid_t qid);
 void rad_sql_cleanup(int type, qid_t qid);
+int rad_sql_checkgroup(AUTH_REQ *req, char *groupname);
+void rad_sql_shutdown();
 
 /* Lower level routines: */
 void rad_sql_connect(int type);
 int rad_sql_reconnect(int type, struct sql_connection *);	
 int rad_sql_query(struct sql_connection *, char *query, int *report_cnt);
 char * rad_sql_getpwd(struct sql_connection *, char *query);
+void * rad_sql_exec(struct sql_connection *conn, char *query);
+char * rad_sql_column(void *data, int ncol);
+int rad_sql_next_tuple(struct sql_connection *conn, void *data);
+void rad_sql_free(struct sql_connection *conn, void *data);
 
 #else
 
 # define rad_sql_check_connect(a)
 # define rad_sql_setup NULL
 # define rad_sql_cleanup NULL
+# define rad_sql_shutdown()
 
 #endif
