@@ -16,7 +16,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
-/* $Id: symtab.h,v 1.2 2000/12/29 11:48:09 gray Exp $ */
+/* $Id: symtab.h,v 1.3 2001/03/12 11:14:16 gray Exp $ */
+
+#ifndef __symtab_h
+#define __symtab_h
 
 typedef struct symbol Symbol;
 struct symbol {
@@ -37,7 +40,9 @@ void symtab_free(Symtab *symtab);
 Symbol * sym_install(Symtab *symtab, char *name);
 Symbol * sym_lookup(Symtab *symtab, char *name);
 Symbol * sym_next(Symbol *sym);
+void symtab_iterate(Symtab *symtab, int (*fn)(), void *closure);
 
 Symbol * alloc_sym(char *, unsigned);
 void     sym_free(Symbol *);
 
+#endif
