@@ -1,5 +1,5 @@
 /* This file is part of GNU RADIUS.
- * Copyright (C) 2000, Sergey Poznyakoff
+ * Copyright (C) 2000,2001, Sergey Poznyakoff
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,17 +16,11 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
-/*
- * auth.c	User authentication.
- *
- *
- * Version:	@(#)auth.c  1.83  21-Mar-1999  miquels@cistron.nl
- *              @(#) $Id: auth.c,v 1.3 2001/01/22 12:33:38 gray Exp $ 
- */
-#define RADIUS_MODULE 4
+
+#define RADIUS_MODULE 6
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: auth.c,v 1.3 2001/01/22 12:33:38 gray Exp $";
+"@(#) $Id: auth.c,v 1.12 2001/03/12 11:20:25 gray Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -62,6 +56,7 @@ static char rcsid[] =
 #if defined(USE_SQL)
 # include <radsql.h>
 #endif
+#include <timestr.h>
 
 #if !defined(__linux__) && !defined(__GLIBC__)
   extern char *crypt();
@@ -69,8 +64,7 @@ static char rcsid[] =
 
 static int pw_expired(UINT4 exptime);
 static int check_disable(char *username, char **user_msg);
-static int check_expiration(VALUE_PAIR *check_item,
-			    char *umsg, char **user_msg);
+static int check_expiration(VALUE_PAIR *check_item, char **user_msg);
 static int unix_pass(char *name, char *passwd);
 
 /*
@@ -103,13 +97,29 @@ pw_expired(exptime)
 	return 0;
 }
 
+/*
+ * Check if the username is valid. Valid usernames consist of 
+ * alphanumeric characters and symbols from valid_char[]
+ * array
+ */
+int
+check_user_name(p)
+	char *p;
+{
+	static char valid_char[] = ".-_";
+	for (; *p && (isalnum(*p) || strchr(valid_char, *p)); p++)
+		;
+	return *p;
+}
+
 int
 check_disable(username, user_msg)
 	char *username;
 	char **user_msg;
 {
 	if (get_deny(username)) {
-		*user_msg = _("Sorry, your account is currently closed\r\n");
+		*user_msg = make_string(
+			    _("Sorry, your account is currently closed\r\n"));
 		return -1;
 	}
 	return 0;
@@ -119,25 +129,25 @@ check_disable(username, user_msg)
  *	Check if account has expired, and if user may login now.
  */
 int
-check_expiration(check_item, umsg, user_msg)
+check_expiration(check_item, user_msg)
 	VALUE_PAIR *check_item;
-	char *umsg;
 	char **user_msg;
 {
 	int result, rc;
 	VALUE_PAIR *pair;
-
-	result = 0;
+	char umsg[80];
+	
+	result = AUTH_OK;
 	if (pair = pairfind(check_item, DA_EXPIRATION)) {
-		rc = pw_expired(check_item->lvalue);
+		rc = pw_expired(pair->lvalue);
 		if (rc < 0) {
-			result = -1;
-			*user_msg = _("Password Has Expired\r\n");
+			result = AUTH_FAIL;
+			*user_msg = make_string(_("Password Has Expired\r\n"));
 		} else if (rc > 0) {
-			sprintf(umsg,
-				_("Password Will Expire in %d Days\r\n"),
-				rc);
-			*user_msg = umsg;
+			radsprintf(umsg, sizeof(umsg),
+				   _("Password Will Expire in %d Days\r\n"),
+				   rc);
+			*user_msg = make_string(umsg);
 		}
 	}
 
@@ -241,11 +251,13 @@ unix_pass(name, passwd)
 /*
  *	Check password.
  *
- *	Returns:	0  OK
- *			-1 Password fail
- *			-2 Rejected
- *			1  End check & return.
+ *	Returns:	AUTH_OK      OK
+ *			AUTH_FAIL    Password fail
+ *                      AUTH_NOUSER  No such user 
+ *			AUTH_REJECT  Rejected
+ *			
  */
+/*ARGSUSED*/
 int
 rad_check_password(authreq, activefd, check_item, namepair,
 		   pw_digest, user_msg, userpass)
@@ -253,7 +265,7 @@ rad_check_password(authreq, activefd, check_item, namepair,
 	int        activefd;
 	VALUE_PAIR *check_item;
 	VALUE_PAIR *namepair;
-	char       *pw_digest;
+	u_char     *pw_digest;
 	char       **user_msg;
 	char       *userpass;
 {
@@ -267,13 +279,10 @@ rad_check_password(authreq, activefd, check_item, namepair,
 	int		auth_type = -1;
 	int		i;
 	int		result;
-
-#ifdef USE_PAM
-        VALUE_PAIR      *pampair;
-	char *pamauth;
-#endif
+        VALUE_PAIR      *p;
+	char            *authdata = NULL;
 	
-	result = 0;
+	result = AUTH_OK;
 	userpass[0] = 0;
 	string[0] = 0;
 
@@ -286,18 +295,16 @@ rad_check_password(authreq, activefd, check_item, namepair,
 		auth_type = auth_type_pair->lvalue;
 
 	if (auth_type == DV_AUTH_TYPE_ACCEPT)
-		return 0;
+		return AUTH_OK;
 
 	if (auth_type == DV_AUTH_TYPE_REJECT) {
 		*user_msg = NULL;
-		return -2;
+		return AUTH_REJECT;
 	}
 
-#ifdef USE_PAM
-        if ((pampair = pairfind(check_item, DA_PAM_AUTH)) != NULL) {
-		pamauth = pampair->strvalue;
+        if ((p = pairfind(check_item, DA_AUTH_DATA)) != NULL) {
+		authdata = p->strvalue;
         }
-#endif
 	
 	/*
 	 *	Find the password sent by the user. It SHOULD be there,
@@ -308,7 +315,7 @@ rad_check_password(authreq, activefd, check_item, namepair,
 	if ((auth_item = pairfind(authreq->request, DA_CHAP_PASSWORD)) == NULL)
 		auth_item = pairfind(authreq->request, DA_PASSWORD);
 	if (auth_item == NULL)
-		return -1;
+		return AUTH_FAIL;
 
 	/*
 	 *	Find the password from the users file.
@@ -323,23 +330,6 @@ rad_check_password(authreq, activefd, check_item, namepair,
 	 */
 	strip_username(1, namepair->strvalue, check_item, name);
 
-	/*
-	 *	For backward compatibility, we check the
-	 *	password to see if it is the magic value
-	 *	UNIX if auth_type was not set.
-	 */
-	if (auth_type < 0 && password_pair) {
-		if (!strcmp(password_pair->strvalue, "UNIX"))
-			auth_type = DV_AUTH_TYPE_SYSTEM;
-		else if (!strcmp(password_pair->strvalue, "PAM"))
-			auth_type = DV_AUTH_TYPE_PAM;
-		else if (!strcmp(password_pair->strvalue, "MYSQL") ||
-			 !strcmp(password_pair->strvalue, "SQL"))
-			auth_type = DV_AUTH_TYPE_MYSQL;
-		else
-			auth_type = DV_AUTH_TYPE_LOCAL;
-	}
-
 	/*
 	 *	Decrypt the password.
 	 */
@@ -357,10 +347,13 @@ rad_check_password(authreq, activefd, check_item, namepair,
 	}
 
 	debug(1,
-		("auth_type=%d, string=%s, namepair=%s, password_pair=%s\n",
+		("auth_type=%d, string=%s, namepair=%s, password_pair=%s",
 		 auth_type, string, name,
 		 password_pair ? password_pair->strvalue : ""));
 
+	if ((p = pairfind(check_item, DA_AUTH_DATA)) != NULL) 
+		authdata = p->strvalue;
+
 	switch (auth_type) {
 		case DV_AUTH_TYPE_SYSTEM:
 			debug(1, ("  auth: System"));
@@ -368,47 +361,47 @@ rad_check_password(authreq, activefd, check_item, namepair,
 			 *	Check the password against /etc/passwd.
 			 */
 			if (unix_pass(name, string) != 0)
-				result = -1;
+				result = AUTH_FAIL;
 			break;
 		case DV_AUTH_TYPE_PAM:
 #ifdef USE_PAM
 			debug(1, ("  auth: Pam"));
 			/*
-			 *	Use the PAM database.
-			 *
-			 *	cjd 19980706 --
-			 *	Use what we found for pamauth or set it to
-			 *	the default "radius" and then jump into
-			 *	pam_pass with the extra info.
+			 * Provide defaults for authdata
 			 */
-			pamauth = pamauth ? pamauth : PAM_DEFAULT_TYPE;
-			if (pam_pass(name, string, pamauth) != 0)
-				result = -1;
+			if (authdata == NULL &&
+			    (p = pairfind(check_item, DA_PAM_AUTH)) != NULL) {
+				authdata = p->strvalue;
+			}
+			authdata = authdata ? authdata : PAM_DEFAULT_TYPE;
+			if (pam_pass(name, string, authdata, user_msg) != 0)
+				result = AUTH_FAIL;
 #else
-			radlog(L_ERR, _("%s: PAM authentication not available"),
-				name);
-			result = -1;
+			radlog(L_ERR,
+			       _("%s: PAM authentication not available"),
+			       name);
+			result = AUTH_NOUSER;
 #endif
 			break;
 		case DV_AUTH_TYPE_MYSQL:
 #ifdef USE_SQL
-			if (rad_sql_pass(authreq, string) != 0)
-				result = -1;
+			result = rad_sql_pass(authreq, authdata, string);
 #else
-			radlog(L_ERR, _("%s: MYSQL authentication not available"),
-				name);
-			result = -1;
+			radlog(L_ERR,
+			       _("%s: MYSQL authentication not available"),
+			       name);
+			result = AUTH_NOUSER;
 #endif
 			break;
 		case DV_AUTH_TYPE_CRYPT_LOCAL:
 			debug(1, ("  auth: Crypt"));
 			if (password_pair == NULL) {
-				result = string[0] ? -1 : 0;
+				result = string[0] ? AUTH_FAIL : AUTH_OK;
 				break;
 			}
 			if (strcmp(password_pair->strvalue,
 			    md5crypt(string, password_pair->strvalue)) != 0)
-					result = -1;
+				result = AUTH_FAIL;
 			break;
 		case DV_AUTH_TYPE_LOCAL:
 			debug(1, ("  auth: Local"));
@@ -421,7 +414,7 @@ rad_check_password(authreq, activefd, check_item, namepair,
 				 */
 				if (password_pair == NULL ||
 				    strcmp(password_pair->strvalue, string)!=0)
-					result = -1;
+					result = AUTH_FAIL;
 				break;
 			}
 
@@ -436,7 +429,7 @@ rad_check_password(authreq, activefd, check_item, namepair,
 			 */
 			strcpy(string, "{chap-password}");
 			if (password_pair == NULL) {
-				result= -1;
+				result= AUTH_FAIL;
 				break;
 			}
 			i = 0;
@@ -459,25 +452,26 @@ rad_check_password(authreq, activefd, check_item, namepair,
 				memcpy(ptr, authreq->vector, AUTH_VECTOR_LEN);
 				i += AUTH_VECTOR_LEN;
 			}
-			md5_calc(pw_digest, string, i);
+			md5_calc(pw_digest, (u_char*) string, i);
 
 			/*
 			 *	Compare them
 			 */
 			if (memcmp(pw_digest, auth_item->strvalue + 1,
 					CHAP_VALUE_LENGTH) != 0)
-				result = -1;
+				result = AUTH_FAIL;
 			else
 				strcpy(userpass, password_pair->strvalue);
 			break;
 		default:
-			result = -1;
+			result = AUTH_FAIL;
 			break;
 	}
 
+	/*??
 	if (result < 0)
 		*user_msg = NULL;
-
+	*/
 	return result;
 }
 
@@ -492,6 +486,9 @@ rad_auth_init(authreq, activefd)
 	int       activefd;
 {
 	VALUE_PAIR	*namepair;
+#ifdef USE_SQL
+	VALUE_PAIR	*p;
+#endif
 
 	/*
 	 *	Get the username from the request
@@ -506,7 +503,16 @@ rad_auth_init(authreq, activefd)
 		authfree(authreq);
 		return -1;
 	}
-
+	debug(1,("checking username: %s", namepair->strvalue));
+	if (check_user_name(namepair->strvalue)) {
+		radlog(L_AUTH, _("Malformed username: [%s] (from nas %s)"),
+		       namepair->strvalue,
+		       nas_name2(authreq));
+		stat_inc(auth, authreq->ipaddr, num_bad_req);
+		authfree(authreq);
+		return -1;
+	}
+		
 	/*
 	 *	Verify the client and Calculate the MD5 Password Digest
 	 */
@@ -522,6 +528,10 @@ rad_auth_init(authreq, activefd)
 		return -1;
 	}
 
+#ifdef USE_SQL
+	if (p = create_pair(DA_QUEUE_ID, 0, NULL, (qid_t)authreq))
+		pairadd(&authreq->request, p);
+#endif
 	/*
 	 *	Add any specific attributes for this username.
 	 */
@@ -549,30 +559,13 @@ rad_auth_init(authreq, activefd)
  * Authentication state machine.
  */
 
-typedef struct auth_mach {
-	AUTH_REQ   *req;
-	VALUE_PAIR *user_check;
-	VALUE_PAIR *user_reply;
-	VALUE_PAIR *proxy_pairs;
-	int        activefd;
-	
-	VALUE_PAIR *namepair;
-	VALUE_PAIR *check_pair;
-	VALUE_PAIR *timeout_pair;
-	char       userpass[AUTH_STRING_LEN];
-	char       *user_msg;
-	char       umsg[AUTH_STRING_LEN];
-	
-	char       *clid;
-	int        state;
-} MACH;
-
 enum auth_state {
 	as_init,
 	as_validate,
 	as_service, 
 	as_disable, 
 	as_service_type,
+	as_realmuse,
 	as_simuse, 
 	as_time, 
 	as_ttl, 
@@ -587,17 +580,36 @@ enum auth_state {
 	AS_COUNT
 };
 
-enum {
+enum list_id {
+	L_null,
 	L_req,
 	L_reply,
-	L_check,
+	L_check
 };
 
+typedef struct auth_mach {
+	AUTH_REQ   *req;
+	VALUE_PAIR *user_check;
+	VALUE_PAIR *user_reply;
+	VALUE_PAIR *proxy_pairs;
+	int        activefd;
+	
+	VALUE_PAIR *namepair;
+	VALUE_PAIR *check_pair;
+	VALUE_PAIR *timeout_pair;
+	char       userpass[AUTH_STRING_LEN];
+	char       *user_msg;
+	
+	char       *clid;
+	enum auth_state state;
+} MACH;
+
 static void sfn_init(MACH*);
 static void sfn_validate(MACH*);
 static void sfn_service(MACH*);
 static void sfn_disable(MACH*);
 static void sfn_service_type(MACH*);
+static void sfn_realmuse(MACH*);
 static void sfn_simuse(MACH*);
 static void sfn_time(MACH*);
 static void sfn_ttl(MACH*);
@@ -615,25 +627,28 @@ struct auth_state_s {
 	enum auth_state this;
 	enum auth_state next;
 	int             attr;
-	int             list;
+	enum list_id    list;
 	void            (*sfn)(MACH*);
 };
 
 struct auth_state_s states[] = {
 	as_init,         as_validate,
-	                 0,               0,     sfn_init,
+	                 0,               L_null,     sfn_init,
 
 	as_validate,     as_service,
-	                 0,               0,     sfn_validate,
+	                 0,               L_null,     sfn_validate,
 	
 	as_service,      as_disable,
 	                 DA_SERVICE_TYPE, L_req, sfn_service,
 	
 	as_disable,      as_service_type,
-	                 0,               0,     sfn_disable,
+	                 0,               L_null,     sfn_disable,
 	
-	as_service_type, as_simuse,
+	as_service_type, as_realmuse,
 	                 DA_SERVICE_TYPE, L_reply, sfn_service_type,
+
+	as_realmuse,     as_simuse,
+	                 0,               L_null,     sfn_realmuse, 
 	
 	as_simuse,       as_time,
 	                 DA_SIMULTANEOUS_USE, L_check, sfn_simuse,
@@ -642,10 +657,10 @@ struct auth_state_s states[] = {
 	                 DA_LOGIN_TIME,   L_check, sfn_time,
 	
 	as_ttl,          as_ipaddr,
-	                 0,               0, sfn_ttl,
+	                 0,               L_null, sfn_ttl,
 	
 	as_ipaddr,       as_exec_wait,
-	                 0,               0, sfn_ipaddr,
+	                 0,               L_null, sfn_ipaddr,
 	
 	as_exec_wait,    as_cleanup_cbkid,
 	                 DA_EXEC_PROGRAM_WAIT, L_reply, sfn_exec_wait,
@@ -657,16 +672,16 @@ struct auth_state_s states[] = {
 	                 DA_MENU,         L_reply, sfn_menu,
 	
 	as_ack,          as_exec_nowait,
-	                 0,               0, sfn_ack,
+	                 0,               L_null, sfn_ack,
 	
 	as_exec_nowait,  as_stop,
 	                 DA_EXEC_PROGRAM, L_reply, sfn_exec_nowait,
 	
-	as_stop,         -1,
-	                 0,               0, NULL,
+	as_stop,         as_stop,
+	                 0,               L_null, NULL,
 	
 	as_reject,       as_stop,
-	                 0,               0, sfn_reject,
+	                 0,               L_null, sfn_reject,
 };
 
 int
@@ -741,6 +756,8 @@ rad_authenticate(authreq, activefd)
 	pairfree(m.user_check);
 	pairfree(m.user_reply);
 	pairfree(m.proxy_pairs);
+	if (m.user_msg)
+		free_string(m.user_msg);
 	bzero(m.userpass, sizeof(m.userpass));
 	return 0;
 }
@@ -823,25 +840,44 @@ sfn_validate(m)
 	/*
 	 *	Validate the user
 	 */
-	if ((rc = check_expiration(m->user_check, m->umsg, &m->user_msg)) >= 0) {
+	if ((rc = check_expiration(m->user_check, &m->user_msg)) >= 0) {
 		rc = rad_check_password(authreq, m->activefd,
 					m->user_check,
 					m->namepair, authreq->digest,
 					&m->user_msg, m->userpass);
-		if (rc > 0) {
+
+		if (rc != AUTH_OK) { 
 			stat_inc(auth, authreq->ipaddr, num_rejects);
-			newstate(as_stop);
-			return;
-		}
-		if (rc == -2) {
-			if (p = pairfind(m->user_reply, DA_REPLY_MESSAGE))
-				m->user_msg = p->strvalue;
-		}
+			newstate(as_reject);
+			
+			switch (rc) {
+			case AUTH_REJECT:
+				radlog(L_AUTH,
+			      _("Rejected user: [%s] CLID %s (from nas %s)"),
+				       m->namepair->strvalue,
+				       m->clid,
+				       nas_name2(authreq));
+				return;
+				
+			case AUTH_NOUSER:
+				radlog(L_AUTH,
+			       _("Invalid user: [%s] CLID %s (from nas %s)"),
+				       m->namepair->strvalue,
+				       m->clid,
+				       nas_name2(authreq));
+				return;
+				
+			case AUTH_FAIL:
+				break;
+			}
+		}		
+              /*if (p = pairfind(m->user_reply, DA_REPLY_MESSAGE))
+			m->user_msg = dup_string(p->strvalue);*/
 	}
 
 	pairmove2(&m->user_reply, &m->proxy_pairs, DA_PROXY_STATE);
 
-	if (rc < 0) {
+	if (rc != AUTH_OK) {
 		/*
 		 *	Failed to validate the user.
 		 */
@@ -906,32 +942,53 @@ sfn_service_type(m)
 		radlog(L_AUTH,
 		       "Login rejected [%s]. Authenticate only user.",
 		       m->namepair->strvalue); 
-		m->user_msg = _("\r\nAccess denied\r\n");
+		m->user_msg = make_string(_("\r\nAccess denied\r\n"));
 		newstate(as_reject);
 	}
 }
 
+void
+sfn_realmuse(m)
+	MACH *m;
+{
+	if (!m->req->realm)
+		return;
+	
+	if (rad_check_realm(m->req->realm) == 0)
+		return;
+	m->user_msg = make_string(
+		_("\r\nRealm quota exceeded - access denied\r\n"));
+	radlog(L_AUTH,
+   _("Login failed: [%s]: realm quota exceeded for %s: CLID %s (from nas %s)"),
+	       m->namepair->strvalue,
+	       m->req->realm,
+	       m->clid,
+	       nas_name2(m->req));
+	newstate(as_reject);
+}
+
 void
 sfn_simuse(m)
 	MACH *m;
 {
 	char  name[AUTH_STRING_LEN];
 	int   rc;
-	
+	char  umsg[AUTH_STRING_LEN];
+
 	strip_username(strip_names,
 		       m->namepair->strvalue, m->user_check, name);
 	if ((rc = rad_check_multi(name, m->req->request,
 				  m->check_pair->lvalue)) == 0)
 		return;
-
+	
 	if (m->check_pair->lvalue > 1) {
-		sprintf(m->umsg,
+		radsprintf(umsg, sizeof(umsg),
 	      _("\r\nYou are already logged in %d times  - access denied\r\n"),
 			(int)m->check_pair->lvalue);
-		m->user_msg = m->umsg;
+		m->user_msg = make_string(umsg);
 	} else {
-		m->user_msg =
-	      _("\r\nYou are already logged in - access denied\r\n");
+		m->user_msg = make_string(
+		      _("\r\nYou are already logged in - access denied\r\n"));
 	}
 
 	radlog(L_WARN,
@@ -947,8 +1004,6 @@ VALUE_PAIR *
 timeout_pair(m)
 	MACH *m;
 {
-	VALUE_PAIR *p;
-
 	if (!m->timeout_pair &&
 	    !(m->timeout_pair = pairfind(m->user_reply, DA_SESSION_TIMEOUT))) {
 		m->timeout_pair = create_pair(DA_SESSION_TIMEOUT,
@@ -963,44 +1018,50 @@ sfn_time(m)
 	MACH *m;
 {
 	int rc;
+	time_t t;
+	unsigned rest;
 	
-	rc = timestr_match(m->check_pair->strvalue, time(NULL));
-	if (rc < 0) {
+	time(&t);
+	rc = ts_check(m->check_pair->strvalue, &t, &rest, NULL);
+	if (rc == 1) {
 		/*
 		 *	User called outside allowed time interval.
 		 */
-		m->user_msg =
-			_("You are calling outside your allowed timespan\r\n");
+		m->user_msg = make_string(
+		      _("You are calling outside your allowed timespan\r\n"));
 		radlog(L_ERR,
-		       _("Outside allowed timespan: [%s]"
-			 " (from nas %s) time allowed: %s"),
+       _("Outside allowed timespan: [%s] (from nas %s) time allowed: %s"),
 		       m->namepair->strvalue,
 		       nas_name2(m->req),
 		       m->check_pair->strvalue);
 		newstate(as_reject);
-	} else if (rc > 0) {
+	} else if (rc == 0) {
 		/*
 		 *	User is allowed, but set Session-Timeout.
 		 */
-		timeout_pair(m)->lvalue = rc;
+		timeout_pair(m)->lvalue = rest;
+		debug(2, ("user %s, span %s, timeout %d",
+			  m->namepair->strvalue,
+                          m->check_pair->strvalue,
+                          rest));
 	}
-	/* rc == 0 is Ok */
 }
 
+/*ARGSUSED*/
 void
 sfn_ttl(m)
 	MACH *m;
 {
 #ifdef USE_NOTIFY	
-	int r;
+	long r;
 	if (timetolive(m->namepair->strvalue, &r) == 0) {
 		if (r > 0) {
 			timeout_pair(m)->lvalue = r;
 		} else {
 			radlog(L_AUTH, _("Zero time to live: [%s]"),
 			       m->namepair->strvalue); 
-			m->user_msg =
-			  _("\r\nSorry, your account is currently closed\r\n");
+			m->user_msg = make_string(
+			 _("\r\nSorry, your account is currently closed\r\n"));
 			newstate(as_reject);
 		}
 	}
@@ -1011,11 +1072,37 @@ void
 sfn_ipaddr(m)
 	MACH *m;
 {
-	VALUE_PAIR *p;
+	VALUE_PAIR *p, *tmp, *pp;
+	
 	/* Assign an IP if necessary */
-	if (!pairfind(m->user_reply, DA_FRAMED_IP_ADDRESS) &&
-	    (p = alloc_ip_pair(m->namepair->strvalue, m->req)))
-		pairadd(&m->user_reply, p);
+	if (!pairfind(m->user_reply, DA_FRAMED_IP_ADDRESS)) {
+#if 0
+		/* **************************************************
+		 * Keep it here until IP allocation is ready
+		 */
+		if (p = alloc_ip_pair(m->namepair->strvalue, m->req))
+			pairadd(&m->user_reply, p);
+		else
+#endif	
+		if (p = pairfind(m->req->request, DA_FRAMED_IP_ADDRESS)) {
+			/* termserver hint */
+			pairadd(&m->user_reply, pairdup(p));
+			if (p = pairfind(m->req->request,
+					 DA_ADD_PORT_TO_IP_ADDRESS))
+				pairadd(&m->user_reply, pairdup(p));
+		}
+	}
+	
+	if ((p = pairfind(m->user_reply, DA_FRAMED_IP_ADDRESS)) &&
+	    (tmp = pairfind(m->user_reply, DA_ADD_PORT_TO_IP_ADDRESS)) &&
+	    tmp->lvalue &&
+	    (pp = pairfind(m->req->request, DA_NAS_PORT_ID)))
+		/* NOTE: This only works because IP numbers are stored in
+		 * host order throughout the program.
+		 */
+		p->lvalue += pp->lvalue;
+		
+	pairdelete(&m->user_reply, DA_ADD_PORT_TO_IP_ADDRESS);
 }
 
 void
@@ -1036,8 +1123,8 @@ sfn_exec_wait(m)
 		newstate(as_reject);
 
 		if (!m->user_msg)
-			m->user_msg =
-			      _("\r\nAccess denied (external check failed).");
+			m->user_msg = make_string(
+			     _("\r\nAccess denied (external check failed)."));
 
 		if (log_mode & RLOG_AUTH) {
 			radlog(L_AUTH,
@@ -1090,7 +1177,8 @@ sfn_menu(m)
 	char state_value[MAX_STATE_VALUE];
 		
 	msg = get_menu(m->check_pair->strvalue);
-	sprintf(state_value, "MENU=%s", m->check_pair->strvalue);
+	radsprintf(state_value, sizeof(state_value),
+		   "MENU=%s", m->check_pair->strvalue);
 	send_challenge(m->req, msg, state_value, m->activefd);
 	
 	debug(1,
@@ -1128,7 +1216,7 @@ sfn_ack(m)
 
 	if (timeout_pair(m)) {
 		debug(5,
-			("timeout for [%s] is set to %d sec",
+			("timeout for [%s] is set to %ld sec",
 			 m->namepair->strvalue, timeout_pair(m)->lvalue));
 	}
 }
