@@ -1,5 +1,5 @@
 /* This file is part of GNU RADIUS.
- * Copyright (C) 2000, Sergey Poznyakoff
+ * Copyright (C) 2000,2001, Sergey Poznyakoff
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,10 +16,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
-#define RADIUS_MODULE 5
+#define RADIUS_MODULE 7
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: exec.c,v 1.3 2001/01/22 12:33:38 gray Exp $"; 
+"@(#) $Id: exec.c,v 1.9 2001/03/12 11:49:37 gray Exp $"; 
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -42,10 +42,10 @@ static char rcsid[] =
 #include <pwd.h>
 #include <grp.h>
 #include <string.h>
+#include <syslog.h>
 #include <radiusd.h>
 
 
-
 #define MAXARGS 64
 
 
@@ -77,7 +77,9 @@ radius_exec_program(cmd, request, reply, exec_wait, user_msg)
 	char buffer[RAD_BUFFER_SIZE];
 	struct passwd *pwd;
 	struct group  *grp;
+#if 0
 	int saved_uid, saved_gid;
+#endif
 	
 	if (cmd[0] != '/') {
 		radlog(L_ERR,
@@ -146,6 +148,7 @@ radius_exec_program(cmd, request, reply, exec_wait, user_msg)
 				    _("radius_exec_program(): too many arguments"));
 				return -1;
 			}
+			debug(10, ("argv[%d] = %s", argc, ptr));
 			argv[argc++] = ptr;
 		}
 		argv[argc++] = NULL;
@@ -162,30 +165,41 @@ radius_exec_program(cmd, request, reply, exec_wait, user_msg)
 
 		chdir("/tmp");
 		
+		#if 0
 		saved_uid = geteuid();
 		saved_gid = getegid();
+		#endif
 		
-		radlog(L_NOTICE,
-		       "radius_exec_program(): setting %d:%d",
-		       pwd->pw_uid,grp->gr_gid);
-		if (setegid(grp->gr_gid)) {
-			radlog(L_CRIT|L_PERROR,
-			       _("radius_exec_program(): setegid failed"));
+		if (grp->gr_gid && setgid(grp->gr_gid)) {
+			radlog(L_ERR|L_PERROR,
+			       _("setgid(%d) failed"), pwd->pw_gid);
 		}
-		if (seteuid(pwd->pw_uid)) {
-			radlog(L_CRIT|L_PERROR,
-			       _("radius_exec_program(): seteuid failed"));
+		if (pwd->pw_uid != 0) {
+#if defined(HAVE_SETEUID)
+			if (seteuid(pwd->pw_uid)) 
+				radlog(L_ERR|L_PERROR,
+				       _("seteuid(%d) failed (ruid=%d, euid=%d)"),
+					 pwd->pw_uid, getuid(), geteuid());
+#elif defined(HAVE_SETREUID)
+			if (setreuid(0, pwd->pw_uid)) 
+				radlog(L_ERR|L_PERROR,
+				       _("setreuid(0,%d) failed (ruid=%d, euid=%d)"),
+					 pwd->pw_uid, getuid(), geteuid());
+#else
+# warning "*** NO WAY TO SET EFFECTIVE UID IN radius_exec_program() ***"
+#endif
 		}
-
 		execvp(argv[0], argv);
 
-		seteuid(saved_uid);
-		setegid(saved_gid);
-
-		radlog(L_CRIT|L_PERROR,
-		       _("radius_exec_program(): cannot run %s"),
-		       argv[0]);
-		exit(1);
+		/*
+		 * Report error via syslog: we might not be able
+		 * to restore initial privileges if we were started
+		 * as non-root.
+		 */
+		openlog("radiusd", LOG_PID, LOG_USER);
+		syslog(LOG_ERR, "can't run %s (ruid=%d, euid=%d): %m",
+		       argv[0], getuid(), geteuid());
+		exit(2);
 	}
 
 	/* Parent branch */ 
@@ -234,8 +248,11 @@ radius_exec_program(cmd, request, reply, exec_wait, user_msg)
 
 	if (WIFEXITED(status)) {
 		status = WEXITSTATUS(status);
-		debug(1,
-			("returned: %d", status));
+		debug(1, ("returned: %d", status));
+		if (status == 2) {
+			radlog(L_ERR,
+			       _("can't run external program (reason reported via syslog channel user.err)"));
+		}
 		return status;
 	}
 	radlog(L_ERR, _("radius_exec_program(): abnormal child exit"));
