@@ -1,5 +1,5 @@
 /* This file is part of GNU RADIUS.
- * Copyright (C) 2000, Sergey Poznyakoff
+ * Copyright (C) 2000,2001, Sergey Poznyakoff
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
-#define RADIUS_MODULE 8
+#define RADIUS_MODULE 10
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -25,7 +25,7 @@
 
 #ifndef lint
 static char rcsid[] = 
- "@(#) $Id: notify.c,v 1.3 2001/01/22 12:33:38 gray Exp $";
+ "@(#) $Id: notify.c,v 1.8 2001/03/12 11:49:37 gray Exp $";
 #endif
 
 #include <sys/types.h>
@@ -57,12 +57,11 @@ struct notify_packet {
 Notify notify_cfg;
 int    sockfd;
 
-int _notify( char *login, char *called_id, int what, long *ttl_ptr );
+int _notify( char *login, int what, long *ttl_ptr );
 
 int
-_notify(login, called_id, what, ttl_ptr)
+_notify(login, what, ttl_ptr)
 	char *login;
-	char *called_id;
 	int what;
 	long *ttl_ptr;
 {
@@ -120,7 +119,7 @@ _notify(login, called_id, what, ttl_ptr)
 	sin = (struct sockaddr_in *) & salocal;
 	memset ((char *) sin, '\0', (size_t) length);
 	sin->sin_family = AF_INET;
-	sin->sin_addr.s_addr = INADDR_ANY;
+	sin->sin_addr.s_addr = htonl(myip);
 	sin->sin_port = htons ((unsigned short) 0);
 	if (bind (sockfd, (struct sockaddr *) sin, length) < 0 ||
 	    getsockname (sockfd, (struct sockaddr *) sin, &length) < 0) {
@@ -232,16 +231,15 @@ notify(login, what, ttl_ptr)
 	int what;
 	long *ttl_ptr;
 {
-	return _notify(login, NULL, what, ttl_ptr);
+	return _notify(login, what, ttl_ptr);
 }
 
 int
-notify_acct(login, what, called_id)
+notify_acct(login, what)
 	char *login;
 	int what;
-	char *called_id;
 {
-	return _notify(login, called_id, what, NULL);
+	return _notify(login, what, NULL);
 }
 
 int
