@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: radctl.c,v 1.3 2001/01/22 12:33:38 gray Exp $";
+"@(#) $Id: radctl.c,v 1.6 2001/03/02 16:54:02 gray Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -114,19 +114,21 @@ format_reply(s)
 	*reply_ptr++ = '\r';
 	*reply_ptr++ = '\n';
 	*reply_ptr = 0;
+	return 0;
 }
 
+/*ARGSUSED*/
 int
 cntl_respond(fd, sa, salen, buf, size)
 	int fd;
 	struct sockaddr *sa;
 	int salen;
-	char *buf;
+	u_char *buf;
 	int size;
 {
 	AUTH_REQ           *authreq;
 	VALUE_PAIR         *pair, *namepair;
-	char	   	   pw_digest[AUTH_PASS_LEN];
+	u_char	   	   pw_digest[AUTH_PASS_LEN];
 	struct sockaddr_in *sin = (struct sockaddr_in *) sa;
 	int                reply_code = PW_AUTHENTICATION_REJECT;
 	int                code, rc;
@@ -300,13 +302,8 @@ cntl_respond(fd, sa, salen, buf, size)
 			break;
 		}
 		reply_code = PW_AUTHENTICATION_ACK;
-		sprintf(reply_ptr, _("restart initiated"));
-		rad_send_reply(reply_code, authreq,
-			       NULL, reply_msg, fd);
-		pairfree(user_check);
-		pairfree(user_reply);
-		authfree(authreq);
-		rad_restart();
+		sprintf(reply_ptr, _("restart scheduled"));
+		schedule_restart();
 		break;
 		
 	case CNTL_SHUTDOWN:
@@ -345,7 +342,16 @@ cntl_respond(fd, sa, salen, buf, size)
 		}
 		reply_code = PW_AUTHENTICATION_ACK;
 		break;
-		
+
+	case CNTL_REMARK:
+		pair = pairfind(authreq->request, DA_CLASS);
+		while (pair) {
+			radlog(L_INFO, "REMARK: %s", pair->strvalue);
+			pair = pairfind(pair->next, DA_CLASS);
+		}
+		reply_code = PW_AUTHENTICATION_ACK;
+		break;
+			
 	default:
 		radlog(L_ERR,
 		       _("unknown command `%s' in control packet"),
