@@ -17,29 +17,28 @@
  *
  */
 
-#define RADIUS_MODULE 11
+#define RADIUS_MODULE 13
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: radius.c,v 1.3 2001/01/22 12:33:38 gray Exp $";
+"@(#) $Id: radius.c,v 1.7 2001/03/02 16:54:02 gray Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
-#include	<sys/types.h>
-#include	<sys/socket.h>
-#include	<sys/time.h>
-#include	<netinet/in.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/time.h>
+#include <netinet/in.h>
 
-#include	<stdio.h>
-#include	<stdlib.h>
-#include	<netdb.h>
-#include	<pwd.h>
-#include	<time.h>
-#include	<ctype.h>
-
-#include	<radiusd.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <netdb.h>
+#include <pwd.h>
+#include <time.h>
+#include <ctype.h>
+#include <radiusd.h>
 
 
 /*
@@ -233,7 +232,7 @@ rad_send_reply(code, authreq, oreply, msg, activefd)
 	 */
 	secretlen = strlen(authreq->secret);
 	memcpy(send_buffer + total_length, authreq->secret, secretlen);
-	md5_calc(digest, (char *)auth, total_length + secretlen);
+	md5_calc(digest, (u_char *)auth, total_length + secretlen);
 	memcpy(auth->vector, digest, AUTH_VECTOR_LEN);
 	memset(send_buffer + total_length, 0, secretlen);
 
@@ -312,7 +311,7 @@ calc_acctdigest(digest, authreq)
 	int	secretlen;
 	CLIENT	*cl;
 	char zero[AUTH_VECTOR_LEN];
-	char	* recvbuf = authreq->data;
+	u_char	* recvbuf = (u_char*) authreq->data;
 	int	len = authreq->data_len;
 
 	/*
@@ -355,7 +354,6 @@ calc_acctdigest(digest, authreq)
 	return memcmp(digest, authreq->vector, AUTH_VECTOR_LEN) ? 2 : 0;
 }
 
-
 /*
  *	Receive UDP client requests, build an authorization request
  *	structure, and attach attribute-value pairs contained in
@@ -474,8 +472,10 @@ radrecv(host, udp_port, buffer, length)
 				pair->strvalue = alloc_string(attrlen + 1);
 				memcpy(pair->strvalue, ptr, attrlen);
 				pair->strvalue[attrlen] = 0;
-				if (debug_on(10))
-					debug_pair("send", pair);
+
+				if (debug_on(10)) 
+					debug_pair("recv", pair);
+
 				if (first_pair == (VALUE_PAIR *)NULL) {
 					first_pair = pair;
 				}
@@ -489,8 +489,10 @@ radrecv(host, udp_port, buffer, length)
 			case PW_TYPE_IPADDR:
 				memcpy(&lval, ptr, sizeof(UINT4));
 				pair->lvalue = ntohl(lval);
-				if (debug_on(10))
-					debug_pair("send", pair);
+
+				if (debug_on(10)) 
+					debug_pair("recv", pair);
+
 				if (first_pair == (VALUE_PAIR *)NULL) {
 					first_pair = pair;
 				}
@@ -597,7 +599,7 @@ send_challenge(authreq, msg, state, activefd)
 	 */
 	secretlen = strlen(authreq->secret);
 	memcpy(send_buffer + total_length, authreq->secret, secretlen);
-	md5_calc(digest, (char *)auth, total_length + secretlen);
+	md5_calc(digest, (u_char *)auth, total_length + secretlen);
 	memcpy(auth->vector, digest, AUTH_VECTOR_LEN);
 	memset(send_buffer + total_length, 0, secretlen);
 
