@@ -1,5 +1,5 @@
 /* This file is part of GNU RADIUS.
- * Copyright (C) 2000, Sergey Poznyakoff
+ * Copyright (C) 2000,2001 Sergey Poznyakoff
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,11 +17,11 @@
  *
  */
 
-#define RADIUS_MODULE 2
+#define RADIUS_MODULE 4
 
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: radiusd.c,v 1.5 2001/01/26 11:48:13 gray Exp $";
+"@(#) $Id: radiusd.c,v 1.13 2001/03/12 13:57:05 gray Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -50,6 +50,8 @@ static char rcsid[] =
 #include <radsql.h>
 #include <log.h>
 #include <symtab.h>
+#include <radutmp.h>
+#include <rewrite.h>
 
 #if defined (sun) && defined(__svr4__)
 RETSIGTYPE (*sun_signal(int signo, void (*func)(int)))(int);
@@ -121,7 +123,7 @@ struct socket_list {
 	struct socket_list *next;
 	int fd;
 	int (*success)(struct sockaddr *, int);
-	int (*respond)(int fd, struct sockaddr *, int, char *, int);
+	int (*respond)(int fd, struct sockaddr *, int, u_char *, int);
 	int (*failure)(struct sockaddr *, int);
 };
 
@@ -131,32 +133,35 @@ static void add_socket_list(int fd, int (*s)(), int (*r)(), int (*f)());
 static void rad_select();
 
 /* Implementation functions */
-int auth_respond(int fd, struct sockaddr *sa, int salen, char *buf, int size);
+int auth_respond(int fd, struct sockaddr *sa, int salen,
+		 u_char *buf, int size);
 int acct_success(struct sockaddr *sa, int salen);
 int acct_failure(struct sockaddr *sa, int salen);
-int snmp_respond(int fd, struct sockaddr *sa, int salen, char *buf, int size);
-int cntl_respond(int fd, struct sockaddr *sa, int salen, char *buf, int size);
+int snmp_respond(int fd, struct sockaddr *sa, int salen,
+		 u_char *buf, int size);
+int cntl_respond(int fd, struct sockaddr *sa, int salen,
+		 u_char *buf, int size);
 
 /* *************************** Global variables. ************************** */
 
 char			*progname;
 
-int debug_flag; /* can be raised from debugger only */
+int        debug_flag; /* can be raised from debugger only */
 
-static int		foreground;
-static int		spawn_flag;
-int			use_dbm = 0;
-int                     open_acct = 1;
-int                     auth_detail;
-int                     strip_names;
-int                     suspend_flag;
+static int foreground;
+static int spawn_flag;
+int	   use_dbm = 0;
+int        open_acct = 1;
+int        auth_detail;
+int        strip_names;
+int        suspend_flag;
 
 Config config = {
 	10,              /* delayed_hup_wait */
 	1,               /* checkrad_assume_logged */
 	MAX_REQUESTS,    /* maximum number of requests */
-	"daemon",        /* exec-program user */
-	"daemon",        /* exec-program group */
+	NULL,            /* exec-program user */
+	NULL,            /* exec-program group */
 };
 
 UINT4			myip;
@@ -194,6 +199,10 @@ static void rad_child_cleanup();
  * This flag means the reload of the configuration is needed
  */
 static int		need_reload = 0;
+/*
+ * When set to 1 radiusd will try to restart itself
+ */
+static int              need_restart = 0;
 
 static void     check_reload();
 
@@ -221,12 +230,11 @@ static int      open_socket(int port, char *type);
 static void reread_config(int reload);
 static UINT4 getmyip(void);
 
-#define OPTSTR "Aa:bd:cfhl:Lnp:Ssvx:yz"
+#define OPTSTR "Aa:bd:fhl:Lm:ni:p:P:Ssvx:yz"
 #ifdef HAVE_GETOPT_LONG
 struct option longopt[] = {
 	"log-auth-detail",    no_argument,       0, 'A',
 	"acct-directory",     required_argument, 0, 'a',
-	"check-config",       no_argument,       0, 'c',
 #ifdef USE_DBM
 	"dbm",                no_argument,       0, 'b',
 #endif
@@ -235,8 +243,11 @@ struct option longopt[] = {
 	"help",               no_argument,       0, 'h', 
 	"logging-directory",  no_argument,       0, 'l',
 	"license",            no_argument,       0, 'L',
+	"mode",               required_argument, 0, 'm',
 	"auth-only",          no_argument,       0, 'n',
+	"ip-address",	      required_argument, 0, 'i',	
 	"port",               required_argument, 0, 'p',
+	"pid-file-dir",       required_argument, 0, 'P',
 	"log-stripped-names", no_argument,       0, 'S',
 	"single-process",     no_argument,       0, 's',
 	"version",            no_argument,       0, 'v',
@@ -251,25 +262,33 @@ struct option longopt[] = {
 # define GETOPT(ac,av,os,lo,li) getopt(ac,av,os)
 #endif
 
+#define MODE_DAEMON    0
+#define MODE_CHECKCONF 1
+#define MODE_TEST      2
+#define MODE_BUILDDBM  3
+
 int  xargc;
 char **xargv;
+char *x_debug_spec;
 
 int
 main(argc, argv)
 	int argc;
 	char **argv;
 {
-	struct	servent		*svp;
-	int			argval;
-	int			t;
-	int                     fd;
-	int			pid;
-	int			radius_port = 0;
-	int                     check_config;    
+	struct servent *svp;
+	int argval;
+	int t;
+	int fd;
+	int pid;
+	int radius_port = 0;
+	int mode = MODE_DAEMON;    
 #ifdef RADIUS_PID
-	FILE			*fp;
+	FILE *fp;
+	char *radpid_dir = RADPID_DIR;
+	char *p;
 #endif
-
+	
 	if ((progname = strrchr(argv[0], '/')) == NULL)
 		progname = argv[0];
 	else
@@ -284,13 +303,18 @@ main(argc, argv)
 		foreground = 0;
 		spawn_flag = 1;
 	}
-	check_config = 0;
 
 	app_setup();
 
 	/* save the invocation */
 	xargc = argc;
 	xargv = argv;
+
+	/*
+	 * Set up some default values
+	 */
+	config.exec_user  = make_string("daemon");
+	config.exec_group = dup_string(config.exec_user);
 	
 	/*
 	 *	Process the options.
@@ -301,31 +325,59 @@ main(argc, argv)
 			auth_detail++;
 			break;
 		case 'a':
-			radacct_dir = optarg;
+			radacct_dir = make_string(optarg);
 			break;
 #ifdef USE_DBM
 		case 'b':
 			use_dbm++;
 			break;
 #endif
-		case 'c':
-			check_config++;
-			break;
 		case 'd':
 			radius_dir = optarg;
 			break;
 		case 'f':
 			foreground = 1;
 			break;
-		case 'l':
-			radlog_dir = optarg;
-			break;
 		case 'L':
 			license();
 			exit(0);
+		case 'l':
+			radlog_dir = make_string(optarg);
+			break;
+		case 'm':
+			switch (optarg[0]) {
+			case 't':
+				mode = MODE_TEST;
+				break;
+			case 'b':
+#ifdef USE_DBM
+				mode = MODE_BUILDDBM;
+#else
+				fprintf(stderr,
+				    _("radiusd compiled without DBM support"));
+				exit(1);
+#endif
+				break;
+			case 'c':
+				mode = MODE_CHECKCONF;
+				break;
+			default:
+				radlog(L_ERR,
+				       _("unknown mode: %s"), optarg);
+			}
+			break;
 		case 'n':
 			open_acct = 0;
 			break;
+		case 'i':
+			if ((myip = get_ipaddr(optarg)) == 0)
+				fprintf(stderr,
+					_("invalid IP address: %s"),
+					optarg);
+			break;
+		case 'P':
+			radpid_dir = optarg;
+			break;
 		case 'p':
 			radius_port = atoi(optarg);
 			break;
@@ -339,6 +391,7 @@ main(argc, argv)
 			version();
 			break;
 		case 'x':
+			x_debug_spec = optarg;
 			set_debug_levels(optarg);
 			break;
 		case 'y':
@@ -356,7 +409,6 @@ main(argc, argv)
 		}
 	}
 
-	radpath_init();
 
 	signal(SIGHUP, sig_hup);
 	signal(SIGUSR1, sig_usr1);
@@ -364,31 +416,23 @@ main(argc, argv)
 	signal(SIGQUIT, sig_fatal);
 	signal(SIGTERM, sig_fatal);
 	signal(SIGCHLD, sig_cleanup);
-#if !defined(MAINTAINER_MODE)
+	signal(SIGBUS, sig_fatal);
 	signal(SIGTRAP, sig_fatal);
 	signal(SIGFPE, sig_fatal);
 	signal(SIGSEGV, sig_fatal);
 	signal(SIGILL, sig_fatal);
-#endif
-#if 0
-	signal(SIGIOT, sig_fatal);
-#endif
+	/* Do not handle SIGIOT, please! */
 
-	if (!foreground && !check_config)
+	if (!foreground && mode != MODE_CHECKCONF)
 		signal(SIGINT, sig_dumpdb);
-
+	else
+		signal(SIGINT, sig_fatal);
+	
 	for (t = 32; t >= 3; t--)
 		close(t);
 
-
-	/*
-	 *	Read config files.
-	 */
-	get_config();
-	stat_init();
-
 	/* 
-	 * Determine port numbers for authentication and accounting
+	 * Determine default port numbers for authentication and accounting
 	 */
 	if (radius_port)
 		auth_port = radius_port;
@@ -399,27 +443,36 @@ main(argc, argv)
 		else
 			auth_port = PW_AUTH_UDP_PORT;
 	}
-	svp = getservbyname ("radacct", "udp");
-	if (radius_port || svp == (struct servent *) 0)
+	if (radius_port ||
+	    (svp = getservbyname ("radacct", "udp")) == (struct servent *) 0)
 		acct_port = auth_port + 1;
 	else
 		acct_port = ntohs(svp->s_port);
-
 	
+	/*
+	 *	Read config files.
+	 */
+	radpath_init();
 	reread_config(0);
-	if (check_config) 
+
+	switch (mode) {
+	case MODE_CHECKCONF:
 		exit(0);
-#if 0
-/*DEBUG ONLY*/
-	test_rewrite();
-	exit(1);
-/*END*/
-#endif
-	
-	if ((myip = getmyip()) == 0) {
+
+	case MODE_TEST:
+		exit(test_shell());
+		
+#ifdef USE_DBM		
+	case MODE_BUILDDBM:
+		exit(builddbm(argv[optind]));
+#endif		
+	}
+
+	if (myip == 0 && (myip = getmyip()) == 0) {
 		radlog(L_CRIT, _("can't find out my own IP address"));
 		exit(1);
 	}
+	radlog(L_INFO, "using %I as my IP address", myip);
 
 	/*
 	 *	Open Authentication socket.
@@ -429,7 +482,7 @@ main(argc, argv)
 
 	if (open_acct) {
 		/*
-		 *	Open Accounting Socket.
+		 * Open Accounting Socket.
 		 */
 		fd = open_socket(acct_port, "acct");
 		add_socket_list(fd, acct_success, auth_respond, acct_failure);
@@ -459,28 +512,30 @@ main(argc, argv)
 		if (pid > 0) {
 			exit(0);
 		}
+		
 #ifdef HAVE_SETSID
 		setsid();
 #endif
 		chdir("/");
 	}
+	/*
+	 * Stop logging to console
+	 */
+	log_disconnect();
+
 	radius_pid = getpid();
 #ifdef RADIUS_PID
-	if ((fp = fopen(RADIUS_PID, "w")) != NULL) {
+	p = mkfilename(radpid_dir, "radiusd.pid");
+	if ((fp = fopen(p, "w")) != NULL) {
 		fprintf(fp, "%d\n", radius_pid);
 		fclose(fp);
 	}
+	efree(p);
 #endif
 
-	/*
-	 *	Use linebuffered or unbuffered stdout if
-	 *	the debug flag is on.
-	 */
-	if (debug_flag)
-		setlinebuf(stdout);
-
 	if (!foreground) {
-		t = open(RADLOG_DIR "/radius.stderr", O_WRONLY);
+		char *p = mkfilename(radlog_dir, "radius.stderr");
+		t = open(p, O_CREAT|O_WRONLY, 0644);
 		if (t != -1) {
 			if (t != 2) 
 				dup2(t, 2);
@@ -491,8 +546,12 @@ main(argc, argv)
 			fflush(stdout);
 			fflush(stderr);
 		}
+		efree(p);
 	}
-
+#ifdef HAVE_SETVBUF
+	setvbuf(stdout, NULL, _IOLBF, 0);
+#endif
+	
 	radlog(L_INFO, _("Ready to process requests."));
 
 	for(;;) {
@@ -567,13 +626,13 @@ rad_select()
 }
 
 /* ************************* Socket queue functions *********************** */
-
+/*ARGSUSED*/
 int
 auth_respond(fd, sa, salen, buf, size)
 	int fd;
 	struct sockaddr *sa;
 	int salen;
-	char *buf;
+	u_char *buf;
 	int size;
 {
 	AUTH_REQ *authreq;
@@ -614,7 +673,7 @@ snmp_respond(fd, sa, salen, buf, size)
 	int fd;
 	struct sockaddr *sa;
 	int salen;
-	char *buf;
+	u_char *buf;
 	int size;
 {
 	struct snmp_req *req;
@@ -666,17 +725,25 @@ reread_config(reload)
 	}
 
 #ifdef USE_SNMP
-	server_stat->auth.status = serv_init;
-	server_stat->acct.status = serv_init;
+	if (server_stat) {
+		server_stat->auth.status = serv_init;
+		server_stat->acct.status = serv_init;
+	}
 #endif	
 
 	/* Read the options */
 	get_config();
-
+	if (!reload) {
+		if (x_debug_spec)
+			set_debug_levels(x_debug_spec);
+		radpath_init();
+		stat_init();
+	}
+	
 	res = reload_config_file(reload_all);
-
+	
 #ifdef USE_SNMP
-
+	
 	server_stat->auth.status = serv_running;
 	snmp_auth_server_reset();
 
@@ -690,6 +757,7 @@ reread_config(reload)
 		       _("Errors reading config file - EXITING"));
 		exit(1);
 	}
+
 }
 
 /*
@@ -704,11 +772,18 @@ getmyip()
 	return get_ipaddr(myname);
 }
 
-
+void
+schedule_restart()
+{
+	need_restart = 1;
+}
 
 void
 check_reload()
 {
+	if (need_restart)
+		rad_restart();
+	
 	if (delayed_hup_time &&
 	    time(NULL) - delayed_hup_time >= config.delayed_hup_wait) {
 		delayed_hup_time = 0;
@@ -1001,7 +1076,7 @@ rad_spawn_child(type, data, activefd)
 	}
 
 	/*
-	 *	This is a new request
+	 * This is a new request
 	 */
 	if (request_count >= config.max_requests) {
 		request_drop(type, data, _("too many requests in queue"));
@@ -1033,7 +1108,7 @@ rad_spawn_child(type, data, activefd)
 	}
 		
 	/*
-	 *	Add this request to the list
+	 * Add this request to the list
 	 */
 	curreq = alloc_entry(sizeof *curreq);
 	curreq->next = NULL;
@@ -1063,7 +1138,7 @@ rad_spawn_child(type, data, activefd)
 	}
 
 	/*
-	 *	fork our child
+	 * fork the child
 	 */
 	if ((child_pid = fork()) < 0) {
 		request_drop(type, data, _("cannot fork"));
@@ -1082,7 +1157,7 @@ rad_spawn_child(type, data, activefd)
 		chdir("/tmp");
 		exit(request_class[type].handler(data, activefd));
 	} else {
-		debug(1, ("started handler at pid %ld", child_pid));
+		debug(1, ("started handler at pid %d", child_pid));
 	}
 
 	/*
@@ -1218,7 +1293,7 @@ stat_request_list(report)
 
 	/* Report the results */
 	for (i = 0; i < NITEMS(request_class); i++) {
-		sprintf(tbuf, "%4.4s  %4d  %4d  %4d",
+		radsprintf(tbuf, sizeof(tbuf), "%4.4s  %4d  %4d  %4d",
 			request_class[i].name,
 			pending_count[i],
 			completed_count[i],
@@ -1317,12 +1392,8 @@ usage()
 #ifdef HAVE_GETOPT_LONG
 "    -A, --log-auth-detail       Do detailed authentication logging.\n"
 "    -a, --acct-directory DIR    Specify accounting directory.\n"
-"    -c, --check-config          Do configuration files syntax check\n"
-"                                and exit.\n"
 #ifdef USE_DBM
-"    -b, --dbm                   Enable DBM support. When used twice,\n"
-"                                allows to use both `users' file and\n"
-"                                DBM database.\n" 
+"    -b, --dbm                   Enable DBM support.\n"
 #endif
 "    -d, --config-directory DIR  Specify alternate configuration directory\n"
 "                                (default " RADIUS_DIR ").\n"
@@ -1330,12 +1401,18 @@ usage()
 "    -L, --license               Display GNU license and exit\n"
 "    -l, --logging-directory DIR Specify alternate logging directory\n"
 "                                (default " RADLOG_DIR ").\n"
+"    -m, --mode {t|c|b}          Select operation mode: test, checkconf,\n"
+"                                builddbm.\n"
 "    -n, --auth-only             Start only authentication process.\n"
-"    -p, --port PORTNO           Use alternate port number.\n" 
+"    -i, --ip-address IP         Use this IP as source address.\n"	
+"    -p, --port PORTNO           Use alternate port number.\n"
+"    -P, --pid-file-dir DIR      Store pidfile in DIR.\n"
+"                                (default " RADPID_DIR ")\n"
 "    -S, --log-stripped-names    Log usernames stripped off any\n"
 "                                prefixes/suffixes.\n"
 "    -s, --single-process        Run in single process mode.\n"
-"    -v, --version               Display program version and exit.\n"
+"    -v, --version               Display program version and configuration\n"
+"                                parameters and exit.\n"
 "    -x, --debug debug_level     Set debugging level.\n"
 "    -y, --log-auth              Log authentications.\n"
 "    -z, --log-auth-pass         Log passwords used.\n"
@@ -1345,9 +1422,7 @@ usage()
 "    -c                          Do configuration files syntax check\n"
 "                                and exit.\n"
 #ifdef USE_DBM
-"    -b                          Enable DBM support. When used twice,\n"
-"                                allows to use both `users' file and\n"
-"                                DBM database.\n" 
+"    -b                          Enable DBM support.\n"
 #endif
 "    -d DIR                      Specify alternate configuration directory\n"
 "                                (default " RADIUS_DIR ").\n"
@@ -1355,12 +1430,18 @@ usage()
 "    -L                          Display GNU license and exit.\n"
 "    -l DIR                      Specify alternate logging directory\n"
 "                                (default " RADLOG_DIR ").\n"
+"    -m {t|c|b}                  Select operation mode: test, checkconf,\n"
+"                                builddbm.\n"
 "    -n                          Start only authentication process.\n"
+"    -i IP                       Use this IP as source address.\n"
 "    -p PORTNO                   Use alternate port number.\n" 
+"    -P DIR                      Store pidfile in DIR.\n"
+"                                (default " RADPID_DIR ")\n"
 "    -S                          Log usernames stripped off any\n"
 "                                prefixes/suffixes.\n"
 "    -s                          Run in single process mode.\n"
-"    -v                          Display program version and exit.\n"
+"    -v                          Display program version and configuration\n"
+"                                parameters and exit.\n"
 "    -x debug_level              Set debugging level.\n"
 "    -y                          Log authentications.\n"
 "    -z                          Log passwords used.\n"
@@ -1381,6 +1462,7 @@ usage()
  *	These values are read from the SERVER_CONFIG part of the
  *	dictionary (of all places!)
  */
+/*UNUSED*/
 int
 config_init()
 {
@@ -1430,14 +1512,16 @@ rad_exit(sig)
 	case 101:
 		radlog(L_CRIT, _("%sfailed in select() - exit."), me);
 		break;
+	case SIGINT:  /* Foreground mode */
 	case SIGTERM:
 		radlog(L_CRIT, _("%sNormal shutdown."), me);
 		break;
 	default:
 		radlog(L_CRIT, _("%sexit on signal (%d)"), me, sig);
-		break;
+		abort();
 	}
 
+	rad_sql_shutdown();
 	exit(sig == SIGTERM ? 0 : 1);
 }
 
@@ -1462,6 +1546,11 @@ rad_restart()
 	struct socket_list *slist;
 	
 	radlog(L_NOTICE, _("restart initiated"));
+	if (xargv[0][0] != '/') {
+		radlog(L_ERR,
+		       _("can't restart: not started as absolute pathname"));
+		return -1;
+	}
 	
 	/* Flush request queues */
 	rad_flush_queues();
@@ -1512,6 +1601,7 @@ rad_restart()
 	execvp(xargv[0], xargv);
 	radlog(L_CRIT|L_PERROR, _("RADIUS NOT RESTARTED: exec failed"));
 	exit(1);
+	/*NOTREACHED*/
 }
 
 
@@ -1544,6 +1634,7 @@ report_info(s)
 	char *s;
 {
 	radlog(L_INFO, "%s", s);
+	return 0;
 }
 
 /*ARGSUSED*/
@@ -1567,7 +1658,7 @@ sig_usr2(sig)
 	int sig;
 {
 	signal(SIGUSR2, sig_usr2);
-	radlog(L_INFO, _("got USR2. Reloading configuration in %ld sec."),
+	radlog(L_INFO, _("got USR2. Reloading configuration in %u sec."),
 	       config.delayed_hup_wait);
 	delayed_hup_time = time(NULL);
 }
@@ -1631,7 +1722,7 @@ open_socket(port, type)
 	sin = (struct sockaddr_in *) & salocal;
         memset ((char *) sin, '\0', sizeof (salocal));
 	sin->sin_family = AF_INET;
-	sin->sin_addr.s_addr = INADDR_ANY;
+	sin->sin_addr.s_addr = htonl(myip);
 	sin->sin_port = htons(port);
 
 	if (bind (fd, & salocal, sizeof (*sin)) < 0) {
@@ -1641,3 +1732,105 @@ open_socket(port, type)
 	return fd;
 }
 
+static char buf[128];
+int doprompt;
+
+char *
+moreinput(buf, bufsize)
+	char *buf;
+	size_t bufsize;
+{
+	if (doprompt)
+		printf("%% ");
+	return fgets(buf, bufsize, stdin);
+}
+
+int
+test_shell()
+{
+	char *tok;
+	int c;
+	NAS *nas;
+	struct radutmp ut;
+	Datatype type;
+	Datum datum;
+	
+#define nextkn() if ((tok = strtok(NULL, " \t")) == NULL) {\
+ printf("arg count\n");\
+ continue; }  
+#define chktkn() if (!tok) { \
+ printf("arg count\n");\
+ continue; }
+
+	printf("** TEST MODE **\n");
+	doprompt = isatty(fileno(stdin));
+	while (tok = moreinput(buf, sizeof(buf))) {
+		while (*tok && isspace(*tok))
+			tok++;
+		c = strlen(tok);
+		if (c > 1 && tok[c-1] == '\n')
+			tok[c-1] = 0;
+		c = *tok++;
+		tok = strtok(tok, " \t");
+		switch (c) {
+		case 0:
+		case '#':
+			continue;
+		case 'h':
+		case '?':
+			printf("h,?                     help\n");
+			printf("q,<EOF>                 quit\n");
+			printf("c LOGIN SID PORT NAS    checkrad\n");
+			printf("r FUNCALL               function call\n");
+			printf("d LEVEL[,LEVEL]         set debug level\n");
+			printf("m                       display memory usage\n"); 
+			break;
+		case 'd':
+			chktkn();
+			set_debug_levels(tok);
+			break;
+		case 'q':
+			return 0;
+		case 'c': /* checkrad */
+			chktkn();
+			strncpy(ut.orig_login, tok, sizeof(ut.orig_login));
+			nextkn();
+			strncpy(ut.session_id, tok, sizeof(ut.session_id));
+			nextkn();
+			ut.nas_port = atoi(tok);
+			nextkn();
+			nas = nas_by_name(tok);
+			if (!nas) {
+				printf("bad nas\n");
+				continue;
+			}
+			ut.nas_address = nas->ipaddr;
+			printf("%d\n", checkrad(nas, &ut));
+			break;
+		case 'r':
+			/* r funcall */
+			chktkn();
+			if (interpret(tok, NULL, &type, &datum))
+				printf("?\n");
+			else {
+				switch (type) {
+				case Integer:
+					printf("%d (%u)", datum.ival,
+					             (unsigned) datum.ival);
+					break;
+				case String:
+					printf("%s", datum.sval);
+					break;
+				}
+				printf("\n");
+			}
+			break;
+		case 'm': /*memory statistics */
+			meminfo(puts);
+			break;
+		default:
+			printf("no command\n");
+		}
+	}
+	return 0;
+}
