@@ -16,11 +16,11 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
-#define RADIUS_MODULE 15
+#define RADIUS_MODULE 16
 
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: radutil.c,v 1.3 2001/01/22 12:33:38 gray Exp $"; 
+"@(#) $Id: radutil.c,v 1.7 2001/03/02 16:54:02 gray Exp $"; 
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -91,13 +91,13 @@ attr_to_str(buf, request, attr)
 		strncpy(tmp, pair->strvalue, AUTH_STRING_LEN);
 		break;
 	case PW_TYPE_INTEGER:
-		sprintf(tmp, "%ld", pair->lvalue);
+		radsprintf(tmp, sizeof(tmp), "%ld", pair->lvalue);
 		break;
 	case PW_TYPE_IPADDR:
 		ipaddr2str(tmp, pair->lvalue);
 		break;
 	case PW_TYPE_DATE:
-		sprintf(tmp, "%ld", pair->lvalue);
+		radsprintf(tmp, sizeof(tmp), "%ld", pair->lvalue);
 		break;
 	default:
 		radlog(L_CRIT,
