@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
-#define RADIUS_MODULE 14
+#define RADIUS_MODULE 15
 #if defined(HAVE_CONFIG_H)
 # include <config.h>
 #endif
@@ -24,7 +24,7 @@
 #ifdef USE_SNMP
 
 #ifndef lint
-static char rcsid[] = "@(#) $Id: snmpserv.c,v 1.3 2001/01/22 12:33:38 gray Exp $";
+static char rcsid[] = "@(#) $Id: snmpserv.c,v 1.8 2001/03/02 16:54:02 gray Exp $";
 #endif
 
 #include <sys/types.h>
@@ -56,7 +56,7 @@ oid * dup_oid(oid *, int);
 oid * snmp_create_oid();
 char * sprint_oid(char *buf, int buflen, oid *name, int len);
 
-int snmp_decode(SNMP_REQ *req, char *buf, int len);
+int snmp_decode(SNMP_REQ *req, u_char *buf, int len);
 int variable_cmp(variable_list *v1, variable_list *v2);
 
 	
@@ -693,7 +693,7 @@ static char		*send_buffer = (char *)i_send_buffer;
 
 SNMP_REQ *
 rad_snmp_respond(buf, len, sa)
-	char *buf;
+	u_char *buf;
 	int len;
 	struct sockaddr_in *sa;
 {
@@ -704,9 +704,9 @@ rad_snmp_respond(buf, len, sa)
 	req->sa = *sa;
 
 	debug(1,
-		("got %d bytes from %s",
+		("got %d bytes from %I",
 		 len,
-		 ipaddr2str(ipbuf, ntohl(req->sa.sin_addr.s_addr))));
+		 ntohl(req->sa.sin_addr.s_addr)));
 	
 	if (snmp_decode(req, buf, len)) {
 		free_entry(req);
@@ -718,7 +718,7 @@ rad_snmp_respond(buf, len, sa)
 int
 snmp_decode(req, buf, len)
 	SNMP_REQ *req;
-	char *buf;
+	u_char *buf;
 	int len;
 {
 	struct snmp_pdu *pdu;
@@ -735,10 +735,10 @@ snmp_decode(req, buf, len)
 		access = check_acl(req->sa.sin_addr.s_addr, community);
 		if (!access) {
 			radlog(L_NOTICE,
-			       _("DENIED attempt to access community %s from %s (%s)"),
+			       _("DENIED attempt to access community %s from %s (%I)"),
 			       community,
 			       ip_hostname(ntohl(req->sa.sin_addr.s_addr)),
-			       ipaddr2str(ipbuf, ntohl(req->sa.sin_addr.s_addr)));
+			       ntohl(req->sa.sin_addr.s_addr));
 			return 1;
 		}
 		req->pdu = pdu;
@@ -747,9 +747,9 @@ snmp_decode(req, buf, len)
 		return 0;
 	} else {
 		char ipbuf[DOTTED_QUAD_LEN];
-		radlog(L_ERR, _("failed SNMP query from %s (%s)"),
+		radlog(L_ERR, _("failed SNMP query from %s (%I)"),
 		    ip_hostname(ntohl(req->sa.sin_addr.s_addr)),
-		    ipaddr2str(ipbuf, ntohl(req->sa.sin_addr.s_addr)));
+		    ntohl(req->sa.sin_addr.s_addr));
 		efree(community);	
 		return 1;
 	}
@@ -778,11 +778,9 @@ snmp_req_drop(type, req, status_str)
 	SNMP_REQ *req;
 	char *status_str;
 {
-	char ipbuf[DOTTED_QUAD_LEN];
-	
 	radlog(L_NOTICE,
-	       _("Dropping SNMP request from client %s: %s"),
-	       ipaddr2str(ipbuf, ntohl(req->sa.sin_addr.s_addr)),
+	       _("Dropping SNMP request from client %I: %s"),
+	       ntohl(req->sa.sin_addr.s_addr),
 	       status_str);
 }
 
@@ -1768,6 +1766,7 @@ get_stat_nasstat(nas, var, ind)
 		break;
 
 	case MIB_KEY_NASLines:
+		stat_count_ports();
 		var->type = SMI_COUNTER32;
 		var->val_len = sizeof(counter);
 		var->val.integer = emalloc(var->val_len);
@@ -1775,6 +1774,7 @@ get_stat_nasstat(nas, var, ind)
 		break;
 
 	case MIB_KEY_NASLinesInUse:
+		stat_count_ports();
 		var->type = SMI_COUNTER32;
 		var->val_len = sizeof(counter);
 		var->val.integer = emalloc(var->val_len);
@@ -1782,6 +1782,7 @@ get_stat_nasstat(nas, var, ind)
 		break;
 
 	case MIB_KEY_NASLinesIdle:
+		stat_count_ports();
 		var->type = SMI_COUNTER32;
 		var->val_len = sizeof(counter);
 		var->val.integer = emalloc(var->val_len);
