@@ -1,5 +1,5 @@
 /* This file is part of GNU RADIUS.
- * Copyright (C) 2000, Sergey Poznyakoff
+ * Copyright (C) 2000,2001, Sergey Poznyakoff
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
  *
  */
 
-#define RADIUS_MODULE 7
+#define RADIUS_MODULE 9
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: sql.c,v 1.3 2001/01/22 12:33:38 gray Exp $";
+"@(#) $Id: sql.c,v 1.11 2001/03/12 11:25:43 gray Exp $";
 #endif
 
 #if defined(HAVE_CONFIG_H)
@@ -52,6 +52,8 @@ static int get_unsigned(char *str, unsigned *retval);
 static char * sql_digest(SQL_cfg *cfg);
 static int sql_digest_comp(char *d1, char *d2);
 static int sql_cfg_comp(SQL_cfg *a, SQL_cfg *b);
+static int chop(char *str);
+static void sql_flush();
 
 SQL_cfg sql_cfg;
 
@@ -69,10 +71,12 @@ SQL_cfg sql_cfg;
 #define STMT_ACCT_STOP_QUERY       12
 #define STMT_ACCT_NASUP_QUERY      13
 #define STMT_ACCT_NASDOWN_QUERY    14
-#define STMT_QUERY_BUFFER_SIZE     15
-#define STMT_IDLE_TIMEOUT          16
-#define STMT_MAX_AUTH_CONNECTIONS  17
-#define STMT_MAX_ACCT_CONNECTIONS  18
+#define STMT_ACCT_KEEPALIVE_QUERY  15
+#define STMT_QUERY_BUFFER_SIZE     16
+#define STMT_IDLE_TIMEOUT          17
+#define STMT_MAX_AUTH_CONNECTIONS  18
+#define STMT_MAX_ACCT_CONNECTIONS  19
+#define STMT_GROUP_QUERY           20
 
 static FILE  *sqlfd;
 static int line_no;
@@ -94,8 +98,11 @@ struct keyword sql_keyword[] = {
 	"auth_db",            STMT_AUTH_DB,
 	"acct_db",            STMT_ACCT_DB,
 	"auth_query",         STMT_AUTH_QUERY,
+	"group_query",        STMT_GROUP_QUERY,
 	"acct_start_query",   STMT_ACCT_START_QUERY,
 	"acct_stop_query",    STMT_ACCT_STOP_QUERY,
+	"acct_alive_query",   STMT_ACCT_KEEPALIVE_QUERY,
+	"acct_keepalive_query", STMT_ACCT_KEEPALIVE_QUERY,
 	"acct_nasup_query",   STMT_ACCT_NASUP_QUERY,
 	"acct_nasdown_query", STMT_ACCT_NASDOWN_QUERY,
 	"query_buffer_size",  STMT_QUERY_BUFFER_SIZE,
@@ -113,6 +120,21 @@ static char *reconnect_file[SQL_NSERVICE] = {
 };
 
 
+/*
+ * Chop off trailing whitespace. Return length of the resulting string
+ */
+int
+chop(str)
+	char *str;
+{
+	int len;
+
+	for (len = strlen(str); len > 0 && isspace(str[len-1]); len--)
+		;
+	str[len] = 0;
+	return len;
+}
+
 char *
 getline()
 {
@@ -409,6 +431,10 @@ rad_sql_init()
 		case STMT_AUTH_QUERY:
 			new_cfg.auth_query = estrdup(cur_ptr);
 			break;
+
+		case STMT_GROUP_QUERY:
+			new_cfg.group_query = estrdup(cur_ptr);
+			break;
 			
 		case STMT_ACCT_START_QUERY:
 			new_cfg.acct_start_query = estrdup(cur_ptr);
@@ -418,6 +444,10 @@ rad_sql_init()
 			new_cfg.acct_stop_query = estrdup(cur_ptr);
 			break;
 			
+		case STMT_ACCT_KEEPALIVE_QUERY:
+			new_cfg.acct_keepalive_query = estrdup(cur_ptr);
+			break;
+			
 		case STMT_ACCT_NASUP_QUERY:
 			new_cfg.acct_nasup_query = estrdup(cur_ptr);
 			break;
@@ -443,15 +473,8 @@ rad_sql_init()
 
 	sql_check_config(&new_cfg);
 
-	if (sql_cfg_comp(&new_cfg, &sql_cfg)) {
-		radlog(L_NOTICE,
-		 _("SQL configuration changed: closing existing connections"));
-
-		rad_flush_queues();
-		
-		close_sql_connections(SQL_AUTH);
-		close_sql_connections(SQL_ACCT);
-	}
+	if (sql_cfg_comp(&new_cfg, &sql_cfg)) 
+		sql_flush();
 
 	/* Free old configuration structure */
 	FREE(sql_cfg.server);
@@ -459,32 +482,52 @@ rad_sql_init()
 	FREE(sql_cfg.password);
 	FREE(sql_cfg.acct_db) ;
 	FREE(sql_cfg.auth_db);
+	FREE(sql_cfg.group_query);
 	FREE(sql_cfg.auth_query);
 	FREE(sql_cfg.acct_start_query);
 	FREE(sql_cfg.acct_stop_query);
 	FREE(sql_cfg.acct_nasup_query);
 	FREE(sql_cfg.acct_nasdown_query);
+	FREE(new_cfg.acct_keepalive_query);
 	FREE(sql_cfg.buf.ptr);
 
 	/* copy new config */
 	sql_cfg = new_cfg;
 	alloc_buffer(&sql_cfg.buf, bufsize);
-	
+		
 	return 0;
 }
 
+void
+rad_sql_shutdown()
+{
+	close_sql_connections(SQL_AUTH);
+	close_sql_connections(SQL_ACCT);
+}
 
 void
 sql_check_config(cfg)
 	SQL_cfg *cfg;
 {
+#define FREE_IF_EMPTY(s) if (s && strcmp(s, "none") == 0) {\
+				efree(s);\
+				s = NULL;\
+			 }
 	/*
 	 * Check if we should do SQL authentication
 	 */
-	if (cfg->doauth && !cfg->auth_query) {
-		radlog(L_ERR,
-		       _("disabling SQL auth: no auth_query specified"));
-		cfg->doauth = 0;
+	if (cfg->doauth) {
+		FREE_IF_EMPTY(cfg->auth_query);
+		if (!cfg->auth_query) {
+			radlog(L_ERR,
+			    _("disabling SQL auth: no auth_query specified"));
+			cfg->doauth = 0;
+		}
+		if (!cfg->group_query) {
+			radlog(L_WARN,
+			       _("SQL auth: no group_query specified"));
+		}
+		FREE_IF_EMPTY(cfg->group_query);
 	}
 	/*
 	 * Check if we should do SQL accounting
@@ -494,25 +537,32 @@ sql_check_config(cfg)
 			radlog(L_WARN,
 			       _("SQL acct: no acct_start_query specified"));
 		}
+		FREE_IF_EMPTY(cfg->acct_start_query);
+
 		if (!cfg->acct_stop_query) {
 			radlog(L_ERR,
 		     _("disabling SQL acct: no acct_stop_query specified"));
 			cfg->doacct = 0;
 		}
+		FREE_IF_EMPTY(cfg->acct_stop_query);
+
 		if (!cfg->acct_nasdown_query) {
 			radlog(L_WARN,
 		     _("SQL acct: no acct_nasdown_query specified"));
 		}
+		FREE_IF_EMPTY(cfg->acct_nasdown_query);
+
 		if (!cfg->acct_nasup_query) {
 			radlog(L_WARN,
 		     _("SQL acct: no acct_nasup_query specified"));
 		}
+		FREE_IF_EMPTY(cfg->acct_nasup_query);
 	}
 	
 	if (cfg->port == 0)
 		cfg->port = RAD_SQL_PORT;
 	
-	radlog(L_INFO, _("SQL init using: %s:%d,%s,%s,%s,%d,%ld,%d,%d"),
+	debug(1, ("SQL init using: %s:%d,%s,%s,%s,%d,%ld,%d,%d",
 	       cfg->server,
 	       cfg->port,
 	       cfg->login,
@@ -521,7 +571,7 @@ sql_check_config(cfg)
 	       cfg->keepopen,
 	       cfg->idle_timeout,
 	       cfg->doacct,
-	       cfg->doauth);
+	       cfg->doauth));
 }
 
 /* ************************************************************************* */
@@ -565,6 +615,20 @@ print_queue()
 	}
 }
 
+void
+sql_flush()
+{
+	if (conn_first != NULL) {
+		radlog(L_NOTICE,
+		 _("SQL configuration changed: closing existing connections"));
+
+		rad_flush_queues();
+		
+		close_sql_connections(SQL_AUTH);
+		close_sql_connections(SQL_ACCT);
+	}
+}
+
 /* Create the new SQL connection and attach it to the head of the
  * list.
  */
@@ -944,6 +1008,19 @@ rad_sql_acct(authreq)
 		break;
 
 	case DV_ACCT_STATUS_TYPE_ALIVE:
+		if (!sql_cfg.acct_keepalive_query)
+			break;
+		query = radius_xlate(sql_cfg.buf.ptr, sql_cfg.buf.size,
+				     sql_cfg.acct_keepalive_query,
+				     authreq->request, NULL);
+		rc = rad_sql_query(conn, query, &count);
+		sqllog(rc, query);
+		if (rc != 0) {
+			radlog(L_INFO,
+			       _("SQL: %d records updated writing keepalive info for NAS %s"),
+			       count,
+			       nas_name2(authreq));
+		}
 		break;
 		
 	}
@@ -954,30 +1031,43 @@ rad_sql_acct(authreq)
 
 
 int
-rad_sql_pass(req, passwd)
+rad_sql_pass(req, authdata, passwd)
 	AUTH_REQ *req;
+	char *authdata;
 	char *passwd;
 {
 	int   rc;
 	char *mysql_passwd;
 	struct sql_connection *conn;
-	
+
 	if (sql_cfg.doauth == 0) {
 		radlog(L_ERR,
 		       _("SQL Auth specified in users file, but not in sqlserver file"));
-		return -1;
+		return AUTH_FAIL;
+	}
+	
+	if (authdata) {
+		pairadd(&req->request,
+			create_pair(DA_AUTH_DATA,
+				    strlen(authdata),
+				    authdata, 0));
 	}
 	radius_xlate(sql_cfg.buf.ptr, sql_cfg.buf.size,
 		     sql_cfg.auth_query,
 		     req->request, NULL);
-
+	pairdelete(&req->request, DA_AUTH_DATA);
+	
 	conn = attach_sql_connection(SQL_AUTH, (qid_t)req);
 	mysql_passwd = rad_sql_getpwd(conn, sql_cfg.buf.ptr);
 	
 	if (!mysql_passwd) {
-		rc = -1;
+		rc = AUTH_NOUSER;
 	} else {
-		rc = strcmp(mysql_passwd, md5crypt(passwd, mysql_passwd));
+		chop(mysql_passwd);
+		if (strcmp(mysql_passwd, md5crypt(passwd, mysql_passwd)) == 0)
+			rc = AUTH_OK;
+		else
+			rc = AUTH_FAIL;
 		efree(mysql_passwd);
 	}
 	
@@ -987,4 +1077,38 @@ rad_sql_pass(req, passwd)
 	return rc;
 }
 
+int
+rad_sql_checkgroup(req, groupname)
+	AUTH_REQ *req;
+	char *groupname;
+{
+	int   rc = -1;
+	struct sql_connection *conn;
+	void *data;
+	char *p;
+	
+	if (sql_cfg.group_query == NULL) 
+		return -1;
+
+	radius_xlate(sql_cfg.buf.ptr, sql_cfg.buf.size,
+		     sql_cfg.group_query,
+		     req->request, NULL);
+
+	conn = attach_sql_connection(SQL_AUTH, (qid_t)req);
+	data = rad_sql_exec(conn, sql_cfg.buf.ptr);
+	while (rc != 0 && rad_sql_next_tuple(conn, data) == 0) {
+		if ((p = rad_sql_column(data,0)) == NULL)
+			break;
+		chop(p);
+		if (strcmp(p, groupname) == 0)
+			rc = 0;
+	}
+	rad_sql_free(conn, data);
+	
+	if (!sql_cfg.keepopen)
+		unattach_sql_connection(SQL_AUTH, (qid_t)req);
+	
+	return rc;
+}
+
 #endif
