@@ -17,10 +17,10 @@
  *
  */
 
-#define RADIUS_MODULE 13
+#define RADIUS_MODULE 14
 #ifndef lint
 static char rcsid[] = 
-"@(#) $Id: stat.c,v 1.3 2001/01/22 12:33:38 gray Exp $";
+"@(#) $Id: stat.c,v 1.7 2001/03/02 16:54:02 gray Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -173,24 +173,23 @@ stat_update(ut, status)
 {
 	NAS *nas;
 	PORT_STAT *port;
-	char ipbuf[DOTTED_QUAD_LEN];
 	long dt;
 	
 	nas = nas_find(ntohl(ut->nas_address));
 	if (!nas) {
 		radlog(L_WARN,
-		    _("stat_update(): portno %d: can't find nas for IP %s"),
+		    _("stat_update(): portno %d: can't find nas for IP %I"),
 		    ut->nas_port,
-		    ipaddr2str(ipbuf, ntohl(ut->nas_address)));
+		    ntohl(ut->nas_address));
 		return;
 	}
 
 	port = stat_find_port(nas, ut->nas_port);
 	if (!port) {
 		radlog(L_WARN,
-		    _("stat_update(): port %d not found on NAS %s"),
+		    _("stat_update(): port %d not found on NAS %I"),
 		    ut->nas_port,
-		    ipaddr2str(ipbuf, ntohl(ut->nas_address)));
+		    ntohl(ut->nas_address));
 		return;
 	}
 
