@@ -1,5 +1,5 @@
 /* This file is part of GNU RADIUS.
- * Copyright (C) 2000, Sergey Poznyakoff
+ * Copyright (C) 2000,2001, Sergey Poznyakoff
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] = 
-"@(#) $Id: symtab.c,v 1.2 2000/12/29 11:39:59 gray Exp $";
+"@(#) $Id: symtab.c,v 1.4 2001/03/12 11:26:12 gray Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -78,7 +78,7 @@ sym_install(symtab, name)
 	char *name;
 {
 	Symbol *sp;
-	unsigned int h = hashval(name, symtab->hashsize);
+	unsigned int h = hashval((unsigned char *)name, symtab->hashsize);
 
 	sp = alloc_sym(name, symtab->elsize);
 	sp->next = symtab->sym[h];
@@ -92,7 +92,7 @@ sym_lookup(symtab, name)
 	char *name;
 {
 	Symbol *sp;
-	unsigned h = hashval(name, symtab->hashsize);
+	unsigned h = hashval((unsigned char *)name, symtab->hashsize);
 
 	for (sp = symtab->sym[h]; sp; sp = sp->next) {
 		if (strcmp(sp->name, name) == 0)
@@ -133,7 +133,7 @@ symtab_delete(symtab, sym)
 	Symbol *sym;
 {
 	Symbol *sp, *prev;
-	unsigned h = hashval(sym->name, symtab->hashsize);
+	unsigned h = hashval((unsigned char *)sym->name, symtab->hashsize);
 
 	/*
 	 * Lookup the symbol
@@ -198,4 +198,24 @@ symtab_free(symtab)
 	}
 }
 
+void
+symtab_iterate(symtab, fn, closure)
+	Symtab *symtab;
+	int (*fn)();
+	void *closure;
+{
+	int i;
+	Symbol *sym, *next;
+	
+	for (i = 0; i < symtab->hashsize; i++) {
+		sym = symtab->sym[i];
+		while (sym) {
+			next = sym->next;
+			if ((*fn)(closure, sym))
+				break;
+			sym = next;
+		}
+	}
+}
+
 
