@@ -1,5 +1,5 @@
 /* This file is part of GNU RADIUS.
- * Copyright (C) 2000, Sergey Poznyakoff
+ * Copyright (C) 2000,2001, Sergey Poznyakoff
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,18 +16,17 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
-/* $Id: version.c,v 1.3 2001/01/22 12:33:38 gray Exp $ */
+/* $Id: version.c,v 1.5 2001/03/12 11:49:37 gray Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
-#include        <sys/types.h>
-#include	<stdio.h>
-#include	<radiusd.h>
+#include <sys/types.h>
+#include <stdio.h>
+#include <radiusd.h>
 
 static char *sys_def[] = {
-	/* here are all the system definitions compilation uses */
 #if defined(__alpha)
 	"__alpha",
 #endif
@@ -118,7 +117,6 @@ static char *compile_flag_str[] = {
 #endif
 };
 
-#define NITEMS(a) sizeof(a)/sizeof((a)[0])
 
 char *server_id;
 
@@ -145,7 +143,7 @@ make_server_ident()
 }
 
 /*
- *	Display the revision number for this program
+ * Display the version number and built-in defaults.
  */
 void
 version()
@@ -172,18 +170,24 @@ version()
 		fprintf(stderr, "%s ", compile_flag_str[i]);
 	}
 	fprintf(stderr, "\n");
+	fprintf(stderr, "Ports in use:\n");
+	fprintf(stderr, " AUTH: %d\n", PW_AUTH_UDP_PORT);
+	fprintf(stderr, " ACCT: %d\n", PW_ACCT_UDP_PORT);
 #if defined(USE_MYSQL)
-	fprintf(stderr, "using sql port %d\n", RAD_SQL_PORT);
+	fprintf(stderr, " SQL : %d\n", RAD_SQL_PORT);
 #endif
+	fprintf(stderr, "Paths:\n");
+	fprintf(stderr, _(" configuration directory: %s\n"), RADIUS_DIR);
+	fprintf(stderr, _(" logging directory:       %s\n"), RADLOG_DIR);
+	fprintf(stderr, _(" accounting directory:    %s\n"), RADACCT_DIR);
 #if defined(DENY_SHELL)
-	fprintf(stderr, _("deny shell is %s\n"), DENY_SHELL);
+	fprintf(stderr, _(" deny shell:              %s\n"), DENY_SHELL);
 #endif
-	fprintf(stderr, _("logfiles stored in %s\n"), RADLOG_DIR);
 
 #ifdef RADIUS_PID
-	fprintf(stderr, _("pidfile %s\n"), RADIUS_PID);
+	fprintf(stderr, _(" pidfile:                 %s\n"), RADIUS_PID);
 #else
-	fprintf(stderr, _("no pidfile\n"));
+	fprintf(stderr, _(" no pidfile\n"));
 #endif
 
 	exit(0);
@@ -207,7 +211,7 @@ char license_text[] =
 void
 license()
 {
-	printf("%s: Copyright 1999,2000 Sergey Poznyakoff\n", progname);
+	printf("%s: Copyright 1999,2000,2001 Sergey Poznyakoff\n", progname);
 	printf("\nThis program is part of GNU Radius\n");
 	printf("%s", license_text);
 }
