@@ -17,8 +17,10 @@
  *
  */
 
+#ifndef lint
 static char rcsid[] = 
-"$Id: radlast.c,v 1.1.1.1 2000/12/22 14:02:11 gray Exp $";
+"$Id: radlast.c,v 1.4 2001/03/02 16:54:02 gray Exp $";
+#endif
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -43,7 +45,6 @@ static char rcsid[] =
 #include <sysdep.h>
 #include <radiusd.h>
 #include <radutmp.h>
-#include <nas.h>
 #include <log.h>
 
 #define IP_ADDR_LEN 15
@@ -79,6 +80,12 @@ WTMP *find_wtmp_nas(WTMP *first, struct radutmp *bp);
 WTMP *find_wtmp_nas_port(WTMP *first, struct radutmp *bp);
 WTMP *find_wtmp_nas_port_sid(WTMP *first, struct radutmp *bp);
 
+NAS * nas_find(UINT4 ipaddr);
+NAS * nas_find_by_name(char *name);
+char * nas_name(UINT4 ipaddr);
+
+extern NAS *naslist;
+
 UINT4 host_ip = 0;
 UINT4 nas_ip = 0;
 int port = 0;
@@ -107,10 +114,11 @@ WTMP *login_list;
 /* List of NAS up/down transitions */
 WTMP *nas_updown_list;
 
-#define OPTSTR "?0123456789c:f:h:mn:lLp:st:w"
+#define OPTSTR "?0123456789c:d:f:h:mn:lLp:st:w"
 #ifdef HAVE_GETOPT_LONG
 struct option longopt[] = {
 	"count",              required_argument, 0, 'c',
+	"config-directory",   required_argument, 0, 'd',
 	"file",               required_argument, 0, 'f',
 	"help",               no_argument,       0, '?',
 	"host",               required_argument, 0, 'h',
@@ -161,6 +169,9 @@ main(argc, argv)
 				return 1;
 			}
 			break;
+		case 'd':
+			radius_dir = optarg;
+			break;
 		case 'f':
 			file = optarg;
 			break;
