@@ -18,7 +18,7 @@
  */
 #ifndef lint
 static char rcsid[] = 
-"$Id: client.c,v 1.3 2001/01/22 12:33:38 gray Exp $";
+"$Id: client.c,v 1.6 2001/03/02 16:54:02 gray Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -115,7 +115,8 @@ radclient_send(config, port_type, code, pair)
 	sin = (struct sockaddr_in *) &salocal;
         memset (sin, 0, sizeof (salocal));
 	sin->sin_family = AF_INET;
-	sin->sin_addr.s_addr = INADDR_ANY;
+	sin->sin_addr.s_addr = config->source_ip ?
+		                   htonl(config->source_ip) : INADDR_ANY;
 
 	local_port = 1025;
 	do {
@@ -207,7 +208,7 @@ radclient_build_request(config, server, code, pair)
 	int      attrlen;
 	AUTH_HDR *auth;
 	char     passbuf[AUTH_PASS_LEN];
-	char     md5buf[256];
+	u_char   md5buf[256];
 	char     *ptr, *length_ptr;
 	int      secretlen;
 	int      i;
@@ -239,9 +240,7 @@ radclient_build_request(config, server, code, pair)
 	for (; pair; pair = pair->next) {
 
 		if (radclient_debug) {
-			printf(_("send: "));
-			fprint_attr_val(stdout, pair);
-			printf("\n");
+			radfprintf(stdout, "%10.10s: %A\n", _("send"), pair);
 		}
 
 		/*
@@ -359,7 +358,7 @@ radclient_recv(host, udp_port, secret, vector, buffer, length)
 	memcpy(auth->vector, vector, AUTH_VECTOR_LEN);
 	secretlen = strlen(secret);
 	memcpy(buffer + length, secret, secretlen);
-	md5_calc(calc_digest, (char *)auth, length + secretlen);
+	md5_calc(calc_digest, (unsigned char *)auth, length + secretlen);
 
 	if (memcmp(reply_digest, calc_digest, AUTH_VECTOR_LEN) != 0) {
 		radlog(L_WARN, _("Received invalid reply digest from server"));
@@ -378,7 +377,6 @@ decode_buffer(host, udp_port, buffer, length)
 {
 	u_char		*ptr;
 	AUTH_HDR	*auth;
-	int		totallen;
 	int		attribute;
 	int		attrlen;
 	DICT_ATTR	*attr;
@@ -395,7 +393,6 @@ decode_buffer(host, udp_port, buffer, length)
 	authreq = alloc_request();
 
 	auth = (AUTH_HDR *)buffer;
-	totallen = ntohs(auth->length);
 
 	/*
 	 *	Fill header fields
@@ -483,9 +480,8 @@ decode_buffer(host, udp_port, buffer, length)
 			}
 
 			if (radclient_debug && pair) {
-				printf(_("recv: "));
-				fprint_attr_val(stdout, pair);
-				printf("\n");
+				radfprintf(stdout, "%10.10s: %A\n", 
+						_("recv"), pair);
 			}
 
 		}
@@ -519,7 +515,8 @@ random_vector(vector)
 /* Initialization. */
 
 RADCLIENT *
-radclient_alloc(bufsize)
+radclient_alloc(source_ip, bufsize)
+	UINT4 source_ip;
 	size_t bufsize;
 {
 	RADCLIENT *client;
@@ -527,6 +524,7 @@ radclient_alloc(bufsize)
 	client = emalloc(sizeof *client);
 
 	/* Provide default values */
+	client->source_ip = source_ip;
 	client->timeout = 1;
 	client->retries = 3;
 	client->bufsize = bufsize ? bufsize : 4096;
@@ -551,6 +549,7 @@ radclient_alloc_server(src)
 	server->port[2] = src->port[2];
 	strncpy(server->secret, src->secret, AUTH_PASS_LEN);
 	server->secret[AUTH_PASS_LEN] = 0;
+	return server;
 }
 
 SERVER *
