@@ -17,9 +17,10 @@
  *
  */
 
-/* log.c	Logging module. */
+#ifndef lint
 static char rcsid[] = 
-"$Id: log.c,v 1.2 2000/12/29 11:39:59 gray Exp $";
+"$Id: log.c,v 1.4 2001/03/12 11:49:38 gray Exp $";
+#endif
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -48,16 +49,16 @@ initlog(name)
 		progname = name;
 }
 
-static int do_log(int lvl, int syserr, char *fmt, va_list ap);
+static int vlog(int lvl, int syserr, char *fmt, va_list ap);
 
 int
-do_log(lvl, syserr, fmt, ap)
+vlog(lvl, syserr, fmt, ap)
 	int lvl;
 	int syserr;
 	char *fmt;
 	va_list ap;
 {
-	unsigned char	*s = ":";
+	char	*s = ":";
 	int errnum = errno;
 
 	fprintf(stderr, "%s: ", progname);
@@ -93,6 +94,7 @@ do_log(lvl, syserr, fmt, ap)
 	return 0;
 }
 
+/*PRINTFLIKE2*/
 int
 radlog(lvl, msg, va_alist)
 	int lvl;
@@ -105,7 +107,7 @@ radlog(lvl, msg, va_alist)
 	r = lvl & L_PERROR;
 	lvl &= L_MASK;
 	va_start(ap);
-	do_log(lvl, r, msg, ap);
+	vlog(lvl, r, msg, ap);
 	va_end(ap);
 
 	return 0;
