@@ -23,7 +23,7 @@
 #include <radpaths.h>
 
 #ifndef lint
-static char rcsid[] = "@(#) $Id: radpaths.c,v 1.2 2000/12/29 11:39:59 gray Exp $";
+static char rcsid[] = "@(#) $Id: radpaths.c,v 1.3 2001/02/06 16:29:47 gray Exp $";
 #endif
 
 char	*radius_dir;
@@ -37,13 +37,18 @@ void
 radpath_init()
 {
 	if (!radius_dir)
-		radius_dir = RADIUS_DIR;
+		radius_dir = make_string(RADIUS_DIR);
 	if (!radlog_dir)
-		radlog_dir = RADLOG_DIR;
+		radlog_dir = make_string(RADLOG_DIR);
 	if (!radacct_dir)
-		radacct_dir = RADACCT_DIR;
+		radacct_dir = make_string(RADACCT_DIR);
 
+	efree(radutmp_path);
 	radutmp_path = mkfilename(radlog_dir, RADUTMP);
+
+	efree(radwtmp_path);
 	radwtmp_path = mkfilename(radlog_dir, RADWTMP);
+
+	efree(radstat_path);
 	radstat_path = mkfilename(radlog_dir, RADSTAT);
 }
