@@ -17,8 +17,10 @@
  *
  */
 
+#ifndef lint
 static char rcsid[] = 
-"$Id: util.c,v 1.3 2001/01/22 12:33:38 gray Exp $";
+"$Id: util.c,v 1.6 2001/03/06 16:42:46 gray Exp $";
+#endif
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -119,6 +121,8 @@ pairadd(first, new)
 	VALUE_PAIR **first;
 	VALUE_PAIR *new;
 {
+	if (!new)
+		return;
 	new->next = NULL;
 	pairlistadd(first, new);
 }
@@ -335,6 +339,20 @@ mkfilename3(dir, subdir, name)
 	return p;
 }
 
+
+/*
+ *	Write a whole list of A/V pairs.
+ */
+void 
+fprint_attr_list(fd, pair)
+	FILE *fd;
+	VALUE_PAIR *pair;
+{
+	while (pair) {
+		radfprintf(fd, "    %A\n", pair);
+	}
+}
+
 #if 0
 int
 parse_exec_program(cmd, program, uid, gid)
