@@ -17,8 +17,10 @@
  *
  */
 
+#ifndef lint
 static char rcsid[] = 
-"$Id: radzap.c,v 1.1.1.1 2000/12/22 14:02:11 gray Exp $";
+"$Id: radzap.c,v 1.3 2001/03/02 16:54:02 gray Exp $";
+#endif
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -45,7 +47,7 @@ static char rcsid[] =
 NAS *naslist;
 #define LOCK_LEN sizeof(struct radutmp)
 
-static int write_wtmp(struct radutmp *ut, int status);
+static int write_wtmp(struct radutmp *ut);
 int radzap(UINT4 nasaddr, int port, char *user, time_t t);
 int radzap_wtmp(UINT4 nasaddr, int port, char *user, time_t t);
 static int confirm(struct radutmp *ut);
@@ -88,7 +90,7 @@ main(argc, argv)
 	char 	*nas = NULL;
 	UINT4	ip = 0;
 	time_t	t;
-	char	buf[256];
+	char    *path;
 	char *s;	
 	int force_wtmp = 0;
 
@@ -147,9 +149,10 @@ main(argc, argv)
 	/*
 	 *	Read the "naslist" file.
 	 */
-	sprintf(buf, "%s/%s", RADIUS_DIR, RADIUS_NASLIST);
-	if (read_naslist_file(buf) < 0)
+	path = mkfilename(radius_dir, RADIUS_NASLIST);
+	if (read_naslist_file(path) < 0)
 		exit(1);
+	efree(path);
 
 	if (nas) {
 		/*
@@ -225,7 +228,7 @@ radzap(nasaddr, port, user, t)
 		/*
 		 *	Add a logout entry to the wtmp file.
 		 */
-		write_wtmp(&u, DV_ACCT_STATUS_TYPE_STOP);
+		write_wtmp(&u);
 	}
 	close(fd);
 
@@ -412,9 +415,8 @@ confirm(utp)
 }
 
 int
-write_wtmp(ut, status)
+write_wtmp(ut)
 	struct radutmp *ut;
-	int status;
 {
 	FILE *fp;
 		
