@@ -82,6 +82,8 @@
 #if 0
 #include <mibii.h>
 #include <snmp_dump.h>
+#else
+#define snmp_dump(packet, length, str, addr)
 #endif
 #include <snmp_error.h>
 #include <snmp_vars.h>
@@ -442,7 +444,7 @@ snmp_parse(struct snmp_session * session,
  * On any error, 0 is returned.
  * The pdu is freed by snmp_send() unless a failure occured.
  */
-#if 0
+
 int 
 snmp_send(struct snmp_session *session, struct snmp_pdu *pdu)
 {
@@ -666,17 +668,18 @@ snmp_read(fdset)
  * should be called with that file descriptor set
  *
  * The timeout is the latest time that SNMP can wait for a timeout.  The
- * select should be done with the minimum time between timeout and any other
- * timeouts necessary.  This should be checked upon each invocation of select.
- * If a timeout is received, snmp_timeout should be called to check if the
- * timeout was for SNMP.  (snmp_timeout is idempotent)
+ * select should be done with the minimum time between timeout and any
+ * other timeouts necessary.  This should be checked upon each invocation
+ * of select. If a timeout is received, snmp_timeout should be called to
+ * check if the timeout was for SNMP.  (snmp_timeout is idempotent)
  *
- * Block is 1 if the select is requested to block indefinitely, rather than time out.
- * If block is input as 1, the timeout value will be treated as undefined, but it must
- * be available for setting in snmp_select_info.  On return, if block is true, the value
- * of timeout will be undefined.
+ * Block is 1 if the select is requested to block indefinitely, rather
+ * than time out. If block is input as 1, the timeout value will be treated
+ * as undefined, but it must be available for setting in snmp_select_info.
+ * On return, if block is true, the value of timeout will be undefined.
  *
- * snmp_select_info returns the number of open sockets.  (i.e. The number of sessions open)
+ * snmp_select_info returns the number of open sockets.  (i.e. The number
+ * of sessions open)
  */
 int
 snmp_select_info(numfds, fdset, timeout, block)
@@ -696,6 +699,7 @@ snmp_select_info(numfds, fdset, timeout, block)
      * For each request outstanding, add it's socket to the fdset,
      * and if it is the earliest timeout to expire, mark it as lowest.
      */
+    FD_ZERO(fdset);
     for (slp = Sessions; slp; slp = slp->next) {
 
 	active++;
@@ -882,4 +886,6 @@ snmp_api_stats(void *outP)
     fprintf(out, "LIBSNMP: Session List: %d active, %d have requests pending.\n",
 	active, requests);
 }
-#endif
+
+
+
